@@ -49,6 +49,15 @@ module ellipse
 !   y_t = tr(2,3) + tr(2,1)*x + tr(2,2)*y
 ! 
 ! HISTORY
+! Apr 2019
+!  Added check for very-nearly identical ellipses
+!
+! Jun 2017
+!  Changed root polishing in ell_ell_roots to Maehly's procedure
+!
+! May 2016
+!  Added a warning if root polishing failed to converge in ell_ell_roots
+!
 ! Apr 2016
 ! Added do while loop to root polishing in ell_ell_roots
 ! p.maxted@keele.ac.uk
@@ -111,6 +120,7 @@ integer, parameter, public :: b_ell_four_intersects    = 4
 integer, parameter, public :: b_ell_identical          = 5
 integer, parameter, public :: b_ell_invalid            = 7
 integer, parameter, public :: b_ell_error              = 8
+integer, parameter, public :: b_ell_warn_inaccurate    = 9
 integer, parameter, public :: b_ell_ellipse_inside_circle = 1
 integer, parameter, public :: b_ell_circle_inside_ellipse = 2
 
@@ -139,11 +149,11 @@ phi = ellpar(5)
 if ((a_p <= 0.0d0).or.(b_p<=0.0d0)) then
   ellipse(:) = bad_dble
   return
-end if
+endif
 if (a_p < b_p) then
   call swap(a_p,b_p)
   phi = mod(phi+halfpi,twopi)
-end if
+endif
 
 ! Calculate coefficients of the quadratic curve
 cosphi = cos(phi)
@@ -246,7 +256,7 @@ if (t1 > t2) then
 else
   a_p = t2 
   b_p = t1
-end if
+endif
 
 ! (23)
 if (a == c) then
@@ -254,20 +264,20 @@ if (a == c) then
     phi = 1.5d0*halfpi
   else
     phi = 0.5d0*halfpi
-  end if
+  endif
 else if ((a < c).eqv.(t1 > t2)) then
   if (b == 0.d0) then
     phi = 0.d0
   else
     phi =  acot((a-c)/(2.d0*b))/2.d0
-  end if
+  endif
 else
   if (b == 0.d0) then
     phi = halfpi
   else
     phi = halfpi+ acot((a-c)/(2.d0*b))/2.d0
-  end if
-end if
+  endif
+endif
 
 ! Copy quadratic curve function into ellipse array
 ellipse(i_ell_qcoeff) = qcoeff
@@ -449,8 +459,8 @@ else
   else
     t(1) = 2.d0*cc/(-bb+sqrt(dd))
     t(2) = (-bb + sqrt(dd))/(2.d0*aa)
-  end if
-end if
+  endif
+endif
 return
 end function ell_line_intersect
 
@@ -508,30 +518,35 @@ end function ell_t_to_xy
 
 !-------------------------------------------------------------------------------
 
-function ell_project_ellipsoid(abc, phi, theta) result(ellipse)
+function ell_project_ellipsoid(abc, phi, incl) result(ellipse)
 ! 
-! Ellipse that is the projection in the sky plane of the triaxial ellipsoid 
-! with semi-axes a, b, c and centred on the origin after rotation by 2 Euler
-! angles, phi then theta.
+! Ellipse that is the projection in the sky plane (s,t) of the triaxial 
+! ellipsoid with semi-axes a, b, c and centred on the origin after the
+! following rotation matrix has been applied.
+!
+! [s]   [  sin(phi)            cos(phi)           0         ][x']
+! [t] = [ -cos(phi)cos(incl)   sin(phi)cos(incl)  sin(incl) ][y']
+! [p]   [  cos(phi)sin(incl)  -sin(phi)sin(incl)  cos(incl) ][z']
 !
 ! Input:
-!  abc(3) - semi-axes on x-axis, y-axis and z-axis, resp.
-!  phi    - rotation around z-axis (radians) 
-!  theta  - rotation around new x-axis after rotation by angle phi (radians)
+!  abc(3) - semi-axes on x'-axis, y'-axis and z'-axis, resp.
+!  phi    - angle between the line-of-sight (p-axis, towards observer)
+!           projected in the x'-y' plane and the  x'-axis
+!  incl   - angle between the p-axis and the z'-axis
 !
 ! Output:
 !  Projected ellipse.
 !  All elements set to bad_dble for invalid input.
 !
 implicit none
-double precision, intent(in)  :: abc(3), phi, theta
+double precision, intent(in)  :: abc(3), phi, incl
 !f2py integer, parameter :: n_ell_par = 14
 double precision              :: ellipse(n_ell_par)
 
 ! Local variables
-double precision :: t1, t2, t3, a2, b2, c2
-double precision :: sinthe, costhe, sinphi, cosphi
-double precision :: sin2phi, cos2phi, sin2the, cos2the
+double precision :: t1, t2, t3, t4, t5, t6, t7, a2, b2, c2
+double precision :: sini, cosi, sinphi, cosphi
+double precision :: sin2phi, cos2phi, sin2i, cos2i
 double precision :: qcoeff(6), aa, cc, anorm
 
 if ( minval(abc) <= 0.d0) then
@@ -540,33 +555,37 @@ if ( minval(abc) <= 0.d0) then
 
 else
 
-  sinthe = sin(theta)
-  costhe = cos(theta)
+  sini = sin(incl)
+  cosi = cos(incl)
   sinphi = sin(phi)
   cosphi = cos(phi)
   sin2phi = sinphi**2
   cos2phi = cosphi**2
-  sin2the = sinthe**2
-  cos2the = costhe**2
+  sin2i = sini**2
+  cos2i = cosi**2
   a2 = 1.d0/abc(1)**2
   b2 = 1.d0/abc(2)**2
   c2 = 1.d0/abc(3)**2
-  t3 = sin2the*(sin2phi*a2 + cos2phi*b2) + cos2the*c2
-  t1 = sinphi*cosphi*(a2-b2)/t3
-  t2 = sinthe*costhe*(c2 - sin2phi*a2 - cos2phi*b2)/t3
+  t1 = sinphi*cosphi*(a2-b2)
+  t2 = sini*cosi*(c2 - cos2phi*a2 - sin2phi*b2)
+  t3 = sin2i*(cos2phi*a2 + sin2phi*b2) + cos2i*c2
+  t4 = sin2phi*a2 + cos2phi*b2
+  t5 = cos2phi*cos2i*a2 + sin2phi*cos2i*b2 + sin2i*c2 
+  t6 = t1*sini
+  t7 = -t1*cosi
 
-  aa = (cos2phi*a2+sin2phi*b2)- t1**2*sin2the
-  cc = (cos2the*(sin2phi*a2+cos2phi*b2)+sin2the*c2)-t2**2
-  anorm = max(aa,cc)
+  aa = t6**2/t3 - t4
+  cc = t2**2/t3 - t5
+  anorm = max(abs(aa),abs(cc))
   qcoeff(1) = aa/anorm
-  qcoeff(2) = -t1*(t2*sinthe+costhe)/anorm
+  qcoeff(2) = (t2*t6/t3 - t7)/anorm
   qcoeff(3) = cc/anorm
   qcoeff(4) = 0.d0
   qcoeff(5) = 0.d0
-  qcoeff(6) = -1.d0/anorm
+  qcoeff(6) = 1.d0/anorm
 
   ellipse = ell_init_from_qcoeff(qcoeff)
-end if
+endif
 
 return 
 
@@ -603,7 +622,7 @@ if (present(inverse)) then
   inverse_l = inverse
 else
   inverse_l = .false.
-end if
+endif
 
 a = ellipse(1)
 b = ellipse(2)
@@ -645,7 +664,7 @@ else
   if (det == 0.d0) then
     ellipse_tr(:) = bad_dble
     return
-  end if
+  endif
   p =  tr(2,2)/det
   q = -tr(1,2)/det
   r = -tr(2,1)/det
@@ -669,7 +688,7 @@ else
   qcoeff(5) = (ft + bt*u + ct*v)/anorm
   qcoeff(6) = (g + at*u**2 + ct*v**2 + 2.d0*(bt*u*v + dt*u + ft*v))/anorm
 
-end if
+endif
 
 ellipse_tr = ell_init_from_qcoeff(qcoeff)
 
@@ -755,6 +774,7 @@ function ell_ell_overlap(ellipse_1, ellipse_2, verbose, &
 !  b_ell_identical        = ellipses are identical
 !  b_ell_invalid          = invalid input
 !  b_ell_error            = error during calculation
+!  b_ell_warn_inaccurate  = results may be inaccurate
 !
 ! Notes
 ! (*) - the ellipses may (also) share one or two tangent points in these cases.
@@ -769,7 +789,12 @@ integer, intent(in)  :: verbose
 double precision, optional, intent(out)  :: xy_intersect(2,4)
 double precision, optional, intent(out)  :: t_int_1(4), t_int_2(4)
 
+! Fraction limit on areas to avoid spurious 4-point intersections due to
+! near-tangent points.
+double precision, parameter :: atol = 1.0d-5
+
 ! Local variables
+double precision :: area_tol, separation
 double precision :: xyc_1(2), xyc_2(2)
 double precision :: area, xyqc(2), xy_int(2,4), qline(4), s_t_a, t_1, t_2, tq(4)
 double precision :: xy1(2), xy2(2), theta_1(4), theta_2(4)
@@ -778,6 +803,8 @@ logical  :: l_1, l_2, whichell
 
 ! Initialize
 flags = 0
+! Minimum area for integration
+area_tol = atol*min(ellipse_1(i_ell_area),ellipse_2(i_ell_area))
 
 call ell_ell_intersect(ellipse_1, ellipse_2, verbose-1, intersect_flags, &
                        xy_int, theta_1, theta_2)
@@ -789,33 +816,33 @@ if (present(t_int_2)) t_int_2(:) = theta_2(:)
 if (btest(intersect_flags, b_ell_identical)) then
   if (verbose >= v_debug) then
     print *,'ell_ell_overlap: identical ellipses.'
-  end if
+  endif
   flags = ibset(flags,b_ell_identical)
   area_flags(1) = ellipse_1(i_ell_area)
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_1_inside_2)) then
     flags = ibset(flags,b_ell_1_inside_2)
     area_flags(1) = ellipse_1(i_ell_area)
     area_flags(2) = flags
     return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_2_inside_1)) then
   flags = ibset(flags,b_ell_2_inside_1)
   area_flags(1) = ellipse_2(i_ell_area)
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_no_overlap)) then
   flags = ibset(flags, b_ell_no_overlap)
   area_flags(1) = 0.d0
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_two_intersects)) then
 
@@ -830,7 +857,7 @@ if (btest(intersect_flags, b_ell_two_intersects)) then
     area = ellipse_1(i_ell_area) &
          - ell_sector_area(theta_1(1),theta_1(2),ellipse_1)  &
          + triangle_area(xyc_1,xy_int(1:2,1),xy_int(1:2,2))
-  end if
+  endif
   xy2 = ell_t_to_xy(midangle(theta_2(1),theta_2(2)), ellipse_2)
   l_2 = ell_point_is_inside(xy2,ellipse_1)
   xyc_2(1:2) = [ellipse_2(i_ell_x_c),ellipse_2(i_ell_y_c)]
@@ -841,15 +868,15 @@ if (btest(intersect_flags, b_ell_two_intersects)) then
     area = area + ellipse_2(i_ell_area) &
          - ell_sector_area(theta_2(1),theta_2(2),ellipse_2)  &
          + triangle_area(xyc_2,xy_int(1:2,1),xy_int(1:2,2))
-  end if
+  endif
   if (verbose >= v_debug) then
     print *,'ell_ell_overlap, two intersections, area = ',area
-  end if
+  endif
   area_flags(1) = area
   area_flags(2) = flags
   return
 
-end if
+endif
 
 if (btest(intersect_flags, b_ell_four_intersects)) then
   flags = ibset(flags,b_ell_four_intersects)
@@ -864,6 +891,35 @@ if (btest(intersect_flags, b_ell_four_intersects)) then
   area = 0.5d0*abs( &
           (xy_int(1,k(3))-xy_int(1,k(1)))*(xy_int(2,k(4))-xy_int(2,k(2))) &
         - (xy_int(1,k(4))-xy_int(1,k(2)))*(xy_int(2,k(3))-xy_int(2,k(1))) )
+  ! Can catch a problem here with a near-tangent point producing four
+  ! intersections all very close together
+  if (area < area_tol) then
+    separation = hypot(ellipse_1(i_ell_x_c)-ellipse_2(i_ell_x_c), &
+                       ellipse_1(i_ell_y_c)-ellipse_2(i_ell_y_c) )
+    if (separation >= (ellipse_1(i_ell_a_p)+ellipse_2(i_ell_a_p))) then
+      flags = ibset(0, b_ell_no_overlap)
+      area = 0
+    else 
+      if (ellipse_1(i_ell_area) < ellipse_2(i_ell_area)) then
+        flags = ibset(0,b_ell_1_inside_2)
+        area = ellipse_1(i_ell_area)
+      else
+        flags = ibset(0,b_ell_2_inside_1)
+        area = ellipse_2(i_ell_area)
+      endif
+    endif
+    if (verbose >= v_debug) then
+      print *,'ell_ell_overlap: resolving near-tangent'
+      print *,'ell_ell_overlap: ',ellipse_1
+      print *,'ell_ell_overlap: ',ellipse_2
+      print *,'ell_ell_overlap: ',flags
+      print *,'ell_ell_overlap: ',area
+    endif
+    area_flags(1) = area
+    area_flags(2) = flags
+    return
+  endif
+
   !  For each pair of adjacent intersection points we should either use the 
   ! area of the elliptical arc for  ellipse_1 or ellipse_2.
   !  Determine which ellipse to use for the first pair of intersection points
@@ -894,7 +950,7 @@ if (btest(intersect_flags, b_ell_four_intersects)) then
       else
         area = area + ellipse_1(i_ell_area) &
         - ell_sector_area(theta_1(j0),theta_1(j1),ellipse_1) - s_t_a
-      end if
+      endif
     else 
       s_t_a=triangle_area(xyc_2,xy_int(1:2,j0),xy_int(1:2,j1),signed=.true.)
       if(s_t_a > 0.d0) then
@@ -902,23 +958,23 @@ if (btest(intersect_flags, b_ell_four_intersects)) then
       else
         area = area + ellipse_2(i_ell_area) &
         - ell_sector_area(theta_2(j0),theta_2(j1),ellipse_2) - s_t_a
-      end if
+      endif
 
-    end if
+    endif
     whichell = .not.whichell
   end do
 
   area_flags(1) =  area
   area_flags(2) =  flags
   return
-end if
+endif
 
 if (verbose >= v_error) then
   print *,'ell_ell_overlap : unknown error'
   print *,intersect_flags
   print *,ellipse_1(i_ell_ellpar)
   print *,ellipse_2(i_ell_ellpar)
-end if
+endif
 area_flags(1) =  bad_dble
 area_flags(2) =  b_ell_error
 return
@@ -970,14 +1026,17 @@ double precision, parameter :: sin60 = 0.86602540378443860d0
 double precision :: tr2(2,3) = reshape([ 0.5d0,sin60, &
                                         -sin60,0.5d0, &
                                            0.0d0,   0.0d0], [2,3])
-double precision :: tr3(2,3) = reshape([ sin60,0.5d0, &
-                                        -0.5d0,sin60, &
+!double precision :: tr3(2,3) = reshape([ sin60,0.5d0, &
+!                                        -0.5d0,sin60, &
+!                                           0.0d0,   0.0d0], [2,3])
+double precision :: tr3(2,3) = reshape([ 1.0d0,0.0d0, &
+                                         0.0d0,1.0d0, &
                                            0.0d0,   0.0d0], [2,3])
 integer :: ix, iy, nxroots, nyroots
 integer :: ii, jj, kk, ll
 integer :: npair(3), ntouch, key(3)
 double precision :: xy_touch(2,4)
-logical :: fail, new, l_left, l_right
+logical :: fail, new, l_left, l_right, lx1, lx2
 
 ! Tolerance parameters
 
@@ -985,6 +1044,8 @@ logical :: fail, new, l_left, l_right
 ! intersection as a fraction of the minimum
 ! Also used to remove duplicate intersections.
 double precision, parameter :: xytol = 1.0d-9
+! Tolerance on ellipse parameters to identify identical ellipses
+double precision, parameter :: stol = 1d-12
 
 flags = 0  ! Initialize/clear bit mask flags
 
@@ -994,17 +1055,23 @@ separation = hypot(ellipse_1(i_ell_x_c)-ellipse_2(i_ell_x_c), &
 if (separation >= (ellipse_1(i_ell_a_p)+ellipse_2(i_ell_a_p))) then
   flags = ibset(flags, b_ell_no_overlap)
   return
-end if
+endif
 
 if (separation <= (ellipse_1(i_ell_b_p)-ellipse_2(i_ell_a_p))) then
   flags = ibset(flags, b_ell_2_inside_1)
   return
-end if
+endif
 
 if (separation <= (ellipse_2(i_ell_b_p)-ellipse_1(i_ell_a_p))) then
   flags = ibset(flags, b_ell_1_inside_2)
   return
-end if
+endif
+
+if (all(abs(ellipse_2(i_ell_ellpar)-ellipse_1(i_ell_ellpar)) < stol)) then
+  flags = ibset(flags, b_ell_identical)
+  return
+endif
+
 
 do ii = 1,3
   ! Find x-/y-values of intersection points
@@ -1026,14 +1093,25 @@ do ii = 1,3
     call ell_ell_roots(ellipse_1t, ellipse_2t, yroots,nyroots,yroots=.true.)
   end select
 
-  if ((nxroots == 5).or.(nyroots==5)) then ! Identical ellipses
-    flags = ibset(flags,b_ell_identical)
-    return
-  end if
-
   if (verbose >= v_debug) then
     print *,'ell_ell_intersect: nxroots, nyroots = ',nxroots, nyroots
-  end if
+  endif
+
+
+  if ((nxroots == 5).or.(nyroots == 5)) then ! Identical ellipses
+    flags = ibset(flags,b_ell_identical)
+    return
+  endif
+
+  if ((nxroots == -5).or.(nyroots == -5)) then ! root finding failed.
+    flags = ibset(flags,b_ell_error)
+  endif
+
+  if ((nxroots < 0 ).or.(nyroots < 0 )) then ! roots may be inaccurate
+    nxroots = abs(nxroots)
+    nyroots = abs(nyroots)
+    flags = ibset(flags,b_ell_warn_inaccurate)
+  endif
 
   ! Store every pair-wise combination of x-,y- roots
   npair(ii) = 0 
@@ -1071,7 +1149,7 @@ do jj = ii+1,3
         .or.(distance(xy(1:2,ll,jj),xy_touch(1:2,ix)) < xytol))then
           new = .false.
           exit
-        end if
+        endif
       end do
       if (new) then
         ntouch = ntouch + 1
@@ -1080,17 +1158,17 @@ do jj = ii+1,3
           print *, ellipse_1
           print *, ellipse_2
           stop
-        end if
+        endif
         xy_touch(1:2,ntouch) = 0.5d0*(xy(1:2,kk,ii)+xy(1:2,ll,jj))
-      end if
-    end if
+      endif
+    endif
   end do
   end do
 end do
 end do
 if (verbose >= v_debug) then
   print *,'ell_ell_intersect: ntouch= ',ntouch
-end if
+endif
 
 
 select case (ntouch)
@@ -1101,14 +1179,14 @@ select case (ntouch)
      (ellipse_1(i_ell_area) < ellipse_2(i_ell_area))) then
     flags = ibset(flags,b_ell_1_inside_2)
     return
-  end if
+  endif
 
   xyc_2(1:2) = [ellipse_2(i_ell_x_c),ellipse_2(i_ell_y_c)]
   if (ell_point_is_inside(xyc_2, ellipse_1).and. &
       (ellipse_2(i_ell_area) < ellipse_1(i_ell_area))) then
     flags = ibset(flags,b_ell_2_inside_1)
     return
-  end if
+  endif
 
   flags = ibset(flags, b_ell_no_overlap)
   return
@@ -1138,19 +1216,53 @@ case (2)
     return
   endif
 
+  ! Need to catch a rare case here where one of the mid-points between the
+  ! intersection/tangent points is also a tangent point (or nearly so), in which
+  ! case the test above fails. Double-check here whether the xy_touch(1:2,1:2)
+  ! are tangent points by testing whether points just either side of these
+  ! points are both inside/outside the other ellipse. 
+  t_right = t_1(1)+1.0D-3
+  t_left  = t_1(1)-1.0D-3
+  xy_right = ell_t_to_xy(t_left ,ellipse_1)
+  xy_left  = ell_t_to_xy(t_right,ellipse_1)
+  l_right = ell_point_is_inside(xy_right,ellipse_2)
+  l_left  = ell_point_is_inside(xy_left ,ellipse_2)
+  lx1 = (l_right.neqv.l_left) ! True if xy_touch(1:2,1) is not tangent 
+
+  t_right = t_1(2)+1.0D-3
+  t_left  = t_1(2)-1.0D-3
+  xy_right = ell_t_to_xy(t_left ,ellipse_1)
+  xy_left  = ell_t_to_xy(t_right,ellipse_1)
+  l_right = ell_point_is_inside(xy_right,ellipse_2)
+  l_left  = ell_point_is_inside(xy_left ,ellipse_2)
+  lx2 = (l_right.neqv.l_left) ! True if xy_touch(1:2,2) is not tangent 
+
+  if (lx1.or.lx2) then ! At least one point is a non-tangent intersection
+    ! Both points should be non-tangent, raise warning if not
+    if (lx1.neqv.lx2) then
+      flags = ibset(flags,b_ell_warn_inaccurate)
+    endif
+    xy_intersect(1:2,1:2) = xy_touch(1:2,1:2)
+    if (present(theta_1)) theta_1(1:2) = t_1(1:2)
+    if (present(theta_2)) theta_2(1:2) = t_2(1:2)
+    flags = ibset(flags, b_ell_two_intersects)
+    return
+  endif
+
+
   xyc_1(1:2) = [ellipse_1(i_ell_x_c), ellipse_1(i_ell_y_c)]
   if (ell_point_is_inside(xyc_1, ellipse_2).and. &
   (ellipse_1(i_ell_area) < ellipse_2(i_ell_area))) then
     flags = ibset(flags,b_ell_1_inside_2)
     return
-  end if
+  endif
 
   xyc_2(1:2) = [ellipse_2(i_ell_x_c),ellipse_2(i_ell_y_c)]
   if (ell_point_is_inside(xyc_2, ellipse_1).and. &
   (ellipse_2(i_ell_area) < ellipse_1(i_ell_area))) then
     flags = ibset(flags,b_ell_2_inside_1)
     return
-  end if
+  endif
 
   flags = ibset(flags, b_ell_no_overlap)
   return
@@ -1176,7 +1288,7 @@ case (3) ! One of the points where the ellipses touch is a tangent point
     if (present(theta_1)) theta_1(1:2) = [t_1(key(1)),t_1(key(3))]
     if (present(theta_2)) theta_2(1:2) = [t_2(key(1)),t_2(key(3))]
     return
-  end if
+  endif
 
   t_right = 0.5d0*(t_1(key(1))+t_1(key(3)))+pi
   t_left  = 0.5d0*(t_1(key(1))+t_1(key(2))) 
@@ -1190,7 +1302,7 @@ case (3) ! One of the points where the ellipses touch is a tangent point
     if (present(theta_1)) theta_1(1:2) = [t_1(key(2)),t_1(key(3))]
     if (present(theta_2)) theta_2(1:2) = [t_2(key(2)),t_2(key(3))]
     return
-  end if
+  endif
 
   t_right = 0.5d0*(t_1(key(3))+t_1(key(1)))+pi
   t_left  = 0.5d0*(t_1(key(3))+t_1(key(2))) 
@@ -1204,7 +1316,7 @@ case (3) ! One of the points where the ellipses touch is a tangent point
     if (present(theta_1)) theta_1(1:2) = [t_1(key(1)),t_1(key(2))]
     if (present(theta_2)) theta_2(1:2) = [t_2(key(1)),t_2(key(2))]
     return
-  end if
+  endif
 
   print *,'ell_ell_interect: case(3) error - no tangent found.'
   print *,ellipse_1(i_ell_ellpar)
@@ -1219,12 +1331,12 @@ case (4)
     do ii = 1,4
     theta_1(ii) = ell_xy_to_t(xy_touch(1:2,ii),ellipse_1)
     end do
-  end if
+  endif
   if (present(theta_2)) then
     do ii = 1,4
       theta_2(ii) = ell_xy_to_t(xy_touch(1:2,ii),ellipse_2)
     end do
-  end if
+  endif
   return
 
 case default
@@ -1242,6 +1354,8 @@ contains
   subroutine ell_ell_roots(ellipse_1, ellipse_2, roots, nroot, yroots)
   ! Roots of the polynomial in x or y that correspond to the intersections
   ! of two ellipses.
+  ! Return nroot = -(# of roots) if root-polishing exceeds iteration limit
+  ! Return nroot = -5 of algorithm fails
   double precision, intent(in) :: ellipse_1(n_ell_par) 
   double precision, intent(in) :: ellipse_2(n_ell_par)
   logical, intent(in), optional :: yroots
@@ -1252,23 +1366,28 @@ contains
   double precision, parameter :: ztol = 1.0d-15
 
   ! Converge tolerance on x/y root position
-  double precision, parameter :: xytol = 1.0d-12
+  double precision, parameter :: xytol = 1.0d-9
+  
+  ! Fractional tolerance on sum of successive corrections in Maehly's 
+  ! procedure to identify cases where the algorithm is bouncing around 
+  ! either side of the root.
+  double precision, parameter :: dtol = 1.0d-6
 
   double precision :: s0,s1,s2,s3,s4,s5
   double precision :: t0,t1,t2,t3,t4,t5
   double precision :: d31,d10,d30,d34,d40,d12,d32,d42,d51,d35,d45
   double precision :: e52,e20,e50,e54,e40,e21,e51,e41,e32,e53,e43
-  double precision :: a(5), xreal(4), ximag(4), xcand, x1, x2, d1
-  double complex :: az(5),z1,z2
-  integer :: degree, i, i1, iter
-  integer, parameter :: itmax = 1000
-  logical :: yroots_l, newroot
+  double precision :: a(5), xreal(4), ximag(4)
+  double precision :: xcand, p, p1, x1, x2, s, d1, d2
+  integer :: degree, i, j, iter
+  integer, parameter :: itmax = 64
+  logical :: yroots_l, newroot, root_polishing_failed
 
   if (present(yroots)) then
     yroots_l = yroots
   else
     yroots_l = .false.
-  end if
+  endif
   s0 = ellipse_1(i_ell_qcoeff(6))
   s1 = 2.0D0*ellipse_1(i_ell_qcoeff(4))
   s2 = 2.0D0*ellipse_1(i_ell_qcoeff(5))
@@ -1283,7 +1402,7 @@ contains
   t4 = 2.0D0*ellipse_2(i_ell_qcoeff(2))
   t5 = ellipse_2(i_ell_qcoeff(3))
 
-  if (yroots) then
+  if (yroots_l) then
     ! Coefficients of quartic whose roots are y values of intersections.
     d10 = s1*t0 - s0*t1
     d12 = s1*t2 - s2*t1
@@ -1320,7 +1439,7 @@ contains
     a(3) = e54*(e40+e21) + e52*(e41-e32) - e51**2 - 2.0d0*e53*e50 
     a(4) = e54*e20 + e52*(e40+e21) - 2.0d0*e51*e50
     a(5) = e52*e20 - e50**2
-  end if
+  endif
   ! Find unique real (or nearly-real) roots of these quartics
   degree = 4
   do i = 1,5
@@ -1328,13 +1447,14 @@ contains
       degree = degree -1
     else 
       exit
-    end if
+    endif
   end do
   if (degree == 0) then ! Identical ellipses
     nroot = 5 
     return
-  end if
+  endif
 
+  ! Use Jenkins & Traub algorithm to get candidate roots
   call rpoly(a(5-degree:5),degree,xreal, ximag, fail)
   if (fail) then 
     print *,'ell_ell_roots: rpoly fail, a',fail,a
@@ -1342,54 +1462,60 @@ contains
     print *,ellipse_2
     print *,a
     print *,degree
-    stop
-  end if
+    nroot = -5
+    return
+  endif
 
-  nroot = 0
+  nroot = 0 
+  root_polishing_failed = .false.
   do i = 1, degree
     if(abs(ximag(i)) < ztol) then
       newroot = .true.
       xcand = xreal(i)
-      do i1 = 1,nroot
-        if (xcand == roots(i1)) then
+      do j = 1,nroot
+        if (abs(xcand - roots(j)) <  epsilon(0.0d0)) then
           newroot = .false.
           exit
-        end if
+        endif
       end do
       if (newroot) then
-        nroot = nroot + 1
         x1 = xcand
         x2 = xcand + 1.0d0
         iter = 0 
-        do while (abs(x1-x2) > xytol)
-          d1 = x1*(x1*(x1*(x1*a(1)+a(2))+a(3))+a(4))+a(5)
-          if (d1 == 0.0d0) then ! Polish root using Laguerre's method
+        d1 = not_set_dble
+
+        do while ((abs(x1-x2) > xytol).and.(iter<itmax))
+          p = x1*(x1*(x1*(x1*a(1)+a(2))+a(3))+a(4))+a(5)
+          if (p == 0.0d0) then 
             x2 = x1
-          else
+          else! Polish root using Maehly's method
             iter = iter + 1
-            if (iter > itmax) then
-              print *,'ell_ell_roots: Root polishing step failed'
-              print *,x1,x2,d1
-              print *,az
-              stop
+            if (iter == itmax) then
+              root_polishing_failed = .true.
+              x2 = x1
+            else
+              p1 = x1*(x1*(4.0d0*x1*a(1)+3.0d0*a(2))+2.0d0*a(3))+a(4)
+              s = 0
+              do j = 1, nroot
+                s = s + 1.0d0/(x1-roots(j))
+              end do
+              d2 = d1
+              d1 = p/(p1 - p*s)
+              if (abs(d1+d2) < dtol*abs(d1)) then
+                x2 = x1
+              else
+                x2 = x1 - d1
+              endif
+              call swap(x1,x2)
             endif
-            if (iter == 1) then
-              az(1) = dcmplx(a(5),0.0d0)
-              az(2) = dcmplx(a(4),0.0d0)
-              az(3) = dcmplx(a(3),0.0d0)
-              az(4) = dcmplx(a(2),0.0d0)
-              az(5) = dcmplx(a(1),0.0d0)
-            end if
-            z1 = dcmplx(x1,0.0d0)
-            z2 = laguerre(az,4,z1)
-            x2 = dble(z2)
-            call swap(x1,x2)
-          end if
+          endif
         end do
+        nroot = nroot + 1
         roots(nroot) = x1
-      end if
-    end if
+      endif
+    endif
   end do
+  if (root_polishing_failed) nroot = -nroot
 
   end subroutine ell_ell_roots
 
@@ -1416,6 +1542,7 @@ function ell_ucircle_overlap(ellipse, verbose, xy_intersect, t_ell,  &
 !  b_ell_identical             = ellipses are identical
 !  b_ell_invalid               = invalid input
 !  b_ell_error                 = error during calculation
+!  b_ell_warn_inaccurate       = results may be inaccurate
 !
 ! Notes
 ! (*) - the ellipse and circle may (also) share one or two tangent points in 
@@ -1455,42 +1582,42 @@ if (present(t_circ)) t_circ(:) = theta_c(:)
 if (btest(intersect_flags, b_ell_identical)) then
   if (verbose >= v_debug) then
     print *,'ell_ucircle_overlap: ellipse identical to unit circle.'
-  end if
+  endif
   flags = ibset(flags,b_ell_identical)
   area_flags(1) = pi
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_circle_inside_ellipse)) then
   if (verbose >= v_debug) then
     print *,'ell_ucircle_overlap: unit circle inside ellipse.'
-  end if
+  endif
   flags = ibset(flags,b_ell_circle_inside_ellipse)
   area_flags(1) = pi
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_ellipse_inside_circle)) then
   if (verbose >= v_debug) then
     print *,'ell_ucircle_overlap: unit circle inside ellipse.'
-  end if
+  endif
   flags = ibset(flags,b_ell_ellipse_inside_circle)
   area_flags(1) = ellipse(i_ell_area)
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_no_overlap)) then
   if (verbose >= v_debug) then
     print *,'ell_ucircle_overlap: no overlap.'
-  end if
+  endif
   flags = ibset(flags, b_ell_no_overlap)
   area_flags(1) = 0.d0
   area_flags(2) = flags
   return
-end if
+endif
 
 if (btest(intersect_flags, b_ell_two_intersects)) then
 
@@ -1505,7 +1632,7 @@ if (btest(intersect_flags, b_ell_two_intersects)) then
     area = ellipse(i_ell_area) &
          - ell_sector_area(theta_e(1),theta_e(2),ellipse)  &
          + triangle_area(xyc,xy_int(1:2,1),xy_int(1:2,2))
-  end if
+  endif
   t_2 = midangle(theta_c(1),theta_c(2))
   xy2 = [ cos(t_2), sin(t_2) ]
   l_2 = ell_point_is_inside(xy2,ellipse)
@@ -1513,15 +1640,15 @@ if (btest(intersect_flags, b_ell_two_intersects)) then
     area = area + arc_area(xy_int(1:2,1),xy_int(1:2,2))
   else
     area = area + pi - arc_area(xy_int(1:2,1),xy_int(1:2,2))
-  end if
+  endif
   if (verbose >= v_debug) then
     print *,'ell_ucircle_overlap, two intersections, area = ',area
-  end if
+  endif
   area_flags(1) = area
   area_flags(2) = flags
   return
 
-end if
+endif
 
 if (btest(intersect_flags, b_ell_four_intersects)) then
   flags = ibset(flags,b_ell_four_intersects)
@@ -1565,29 +1692,29 @@ if (btest(intersect_flags, b_ell_four_intersects)) then
       else
         area = area + ellipse(i_ell_area) &
         - ell_sector_area(theta_e(j0),theta_e(j1),ellipse) - s_t_a
-      end if
+      endif
     else 
       s_t_a = xy_int(1,j0)*xy_int(2,j1) - xy_int(1,j1)*xy_int(2,j0)
       if(s_t_a > 0.d0) then
         area = area + arc_area(xy_int(1:2,j0),xy_int(1:2,j1))
       else
         area = area + pi - arc_area(xy_int(1:2,j0),xy_int(1:2,j1))
-      end if
+      endif
 
-    end if
+    endif
     whichell = .not.whichell
   end do
 
   area_flags(1) =  area
   area_flags(2) =  flags
   return
-end if
+endif
 
 if (verbose >= v_error) then
   print *,'ell_ucircle_overlap : unknown error'
   print *,intersect_flags
   print *,ellipse(i_ell_ellpar)
-end if
+endif
 area_flags(1) =  bad_dble
 area_flags(2) =  b_ell_error
 return
