@@ -75,10 +75,12 @@ LOGICAL   :: zerok
 !         indicate the smaller range.
 ! base    the base of the floating-point number system used.
 
-base = RADIX(0.0)
-eta = EPSILON(1.0)
-infin = HUGE(0.0)
-smalno = TINY(0.0)
+base = RADIX(0.0d0)
+eta = EPSILON(1.0d0)
+infin = HUGE(0.0d0)
+smalno = TINY(0.0d0)
+
+temp = 0
 
 ! are and mre refer to the unit error in + and * respectively.
 ! They are assumed to be the same as eta.
@@ -89,8 +91,8 @@ lo = smalno / eta
 ! Initialization of constants for shift rotation
 xx = SQRT(0.5)
 yy = -xx
-cosr = -.069756474
-sinr = .99756405
+cosr = -0.069756474d0
+sinr = 0.99756405d0
 fail = .false.
 n = degree
 nn = n + 1
@@ -100,7 +102,7 @@ IF (op(1) == 0.d0) THEN
   fail = .true.
   degree = 0
   RETURN
-END IF
+endif
 
 ! Remove the zeros at the origin if any
 10 IF (op(nn) == 0.0D0) THEN
@@ -110,7 +112,7 @@ END IF
   nn = nn - 1
   n = n - 1
   GO TO 10
-END IF
+endif
 
 ! Make a copy of the coefficients
 p(1:nn) = op(1:nn)
@@ -124,11 +126,11 @@ p(1:nn) = op(1:nn)
     zeror(degree) = -p(2) / p(1)
     zeroi(degree) = 0.0D0
     RETURN
-  END IF
+  endif
   CALL quad(p(1), p(2), p(3), zeror(degree-1), zeroi(degree-1),  &
             zeror(degree), zeroi(degree))
   RETURN
-END IF
+endif
 
 ! Find largest and smallest moduli of coefficients.
 MAX = 0
@@ -150,12 +152,12 @@ IF (sc <= 1.0) THEN
   IF (sc == 0.) sc = smalno
 ELSE
   IF (infin/sc < MAX) GO TO 60
-END IF
-l = LOG(sc) / LOG(base) + .5
+endif
+l = INT(LOG(sc) / LOG(base) + .5)
 factor = (base*1.0D0) ** l
 IF (factor /= 1.d0) THEN
   p(1:nn) = factor * p(1:nn)
-END IF
+endif
 
 ! compute lower bound on moduli of zeros.
 60 pt(1:nn) = ABS(p(1:nn))
@@ -167,7 +169,7 @@ IF (pt(n) /= 0.) THEN
 ! if newton step at the origin is better, use it.
   xm = -pt(nn) / pt(n)
   IF (xm < x) x = xm
-END IF
+endif
 
 ! chop the interval (0,x) until ff .le. 0
 80 xm = x * .1
@@ -178,7 +180,7 @@ END DO
 IF (ff > 0.) THEN
   x = xm
   GO TO 80
-END IF
+endif
 dx = x
 
 ! do newton iteration until x converges to two decimal places
@@ -193,7 +195,7 @@ dx = x
   dx = ff / df
   x = x - dx
   GO TO 100
-END IF
+endif
 bnd = x
 
 ! compute the derivative as the intial k polynomial
@@ -225,7 +227,7 @@ DO  jj = 1, 5
     END DO
     k(1) = 0.d0
     zerok = k(n) == 0.d0
-  END IF
+  endif
 END DO
 
 ! save k for restarts with new shifts
@@ -263,7 +265,7 @@ DO  cnt = 1, 20
     zeror(j+1) = lzr
     zeroi(j+1) = lzi
     GO TO 30
-  END IF
+  endif
 
 ! If the iteration is unsuccessful another quadratic
 ! is chosen after restoring k
@@ -369,7 +371,7 @@ DO  j = 1, l2
         ui = -(s+s)
         vi = s * s
         GO TO 20
-      END IF
+      endif
 
 ! Restore variables
       50 u = svu
@@ -383,8 +385,8 @@ DO  j = 1, l2
 ! Recompute qp and scalar values to continue the second stage
       CALL quadsd(nn, u, v, p, qp, a, b)
       CALL calcsc(TTYPE)
-    END IF
-  END IF
+    endif
+  endif
   ovv = vv
   oss = ss
   otv = tv
@@ -417,6 +419,8 @@ tried = .false.
 u = uu
 v = vv
 j = 0
+relstp = 0
+omp = 0
 
 ! Main loop
 10 CALL quad(1.d0, u, v, szr, szi, lzr, lzi)
@@ -445,7 +449,7 @@ ee = (5.*mre+4.*are) * ee - (5.*mre+2.*are) * (ABS( DBLE(a)  + t) +  &
 IF (mp <= 20.*ee) THEN
   nz = 2
   RETURN
-END IF
+endif
 j = j + 1
 
 ! Stop iteration after 20 steps
@@ -466,8 +470,8 @@ IF (j >= 2) THEN
     END DO
     tried = .true.
     j = 0
-  END IF
-END IF
+  endif
+endif
 omp = mp
 
 ! Calculate next k polynomial and new u and v
@@ -506,6 +510,8 @@ nz = 0
 s = sss
 iflag = 0
 j = 0
+t = 0
+omp = 0
 
 ! Main loop
 10 pv = p(1)
@@ -532,7 +538,7 @@ IF (mp <= 20.*((are+mre)*ee - mre*mp)) THEN
   szr = s
   szi = 0.d0
   RETURN
-END IF
+endif
 j = j + 1
 
 ! Stop iteration after 10 steps
@@ -544,8 +550,8 @@ IF (j >= 2) THEN
     iflag = 1
     sss = s
     RETURN
-  END IF
-END IF
+  endif
+endif
 
 ! Return if the polynomial value has increased significantly
 omp = mp
@@ -570,7 +576,7 @@ ELSE
   DO  i = 2, n
     k(i) = qk(i-1)
   END DO
-END IF
+endif
 kv = k(1)
 DO  i = 2, n
   kv = kv * s + k(i)
@@ -601,8 +607,8 @@ IF (ABS(c) <= ABS(k(n))*100.*eta) THEN
     TTYPE = 3
 ! type=3 indicates the quadratic is almost a factor of k
     RETURN
-  END IF
-END IF
+  endif
+endif
 
 IF (ABS(d) >= ABS(c)) THEN
   TTYPE = 2
@@ -615,7 +621,7 @@ IF (ABS(d) >= ABS(c)) THEN
   a1 = b * f - a
   a7 = (f+u) * a + h
   RETURN
-END IF
+endif
 TTYPE = 1
 ! type=1 indicates that all formulas are divided by c
 e = a / c
@@ -651,7 +657,7 @@ IF (TTYPE /= 3) THEN
       k(i) = a3 * qk(i-2) - a7 * qp(i-1)
     END DO
     RETURN
-  END IF
+  endif
 
 ! Use scaled form of the recurrence
   a7 = a7 / a1
@@ -662,7 +668,7 @@ IF (TTYPE /= 3) THEN
     k(i) = a3 * qk(i-2) - a7 * qp(i-1) + qp(i)
   END DO
   RETURN
-END IF
+endif
 
 ! Use unscaled form of the recurrence if type is 3
 k(1) = 0.d0
@@ -695,7 +701,7 @@ IF (TTYPE /= 3) THEN
   ELSE
     a4 = (a+g) * f + h
     a5 = (f+u) * c + v * d
-  END IF
+  endif
 
 ! Evaluate new quadratic coefficients.
   b1 = -k(n) / p(nn)
@@ -709,8 +715,8 @@ IF (TTYPE /= 3) THEN
     uu = u - (u*(c3+c2)+v*(b1*a1+b2*a7)) / temp
     vv = v * (1.+c4/temp)
     RETURN
-  END IF
-END IF
+  endif
+endif
 
 ! If type=3 the quadratic is zeroed
 uu = 0.d0
@@ -773,7 +779,7 @@ RETURN
   sr = 0.d0
   lr = -b1 / a
   GO TO 10
-END IF
+endif
 
 ! Compute discriminant avoiding overflow
 b = b1 / 2.d0
@@ -785,7 +791,7 @@ ELSE
   IF (c < 0.d0) e = -a
   e = b * (b/ABS(c)) - e
   d = SQRT(ABS(e)) * SQRT(ABS(c))
-END IF
+endif
 IF (e >= 0.d0) THEN
 
 ! Real zeros
@@ -794,7 +800,7 @@ IF (e >= 0.d0) THEN
   sr = 0.d0
   IF (lr /= 0.d0) sr = (c/lr) / a
   GO TO 10
-END IF
+endif
 ! complex conjugate zeros
 sr = -b / a
 lr = sr
