@@ -230,6 +230,7 @@ double precision function spot_limb_eclipse(ellipse, ellipse_s, xy_tng, &
                                             large, verbose)
 ! Fraction of a spot on the limb of a star that is eclipsed by an ellipse.
 !
+! Returns bad_dble is algorithm fails
 ! All input angles are in radians
 !
 implicit none
@@ -241,7 +242,7 @@ double precision, intent(in) :: xy_tng(2,2)
 logical, intent(in) :: large                       ! See below ...
 integer, intent(in) :: verbose
 
-! A circular spot on the limb in prjection is seen as an ellipse that is
+! A circular spot on the limb in projection is seen as an ellipse that is
 ! interior to the circle and tangent to it at two points.
 !
 ! large  = .true. to take the larger lens-shaped area between the tangent
@@ -277,9 +278,9 @@ spot_limb_eclipse = bad_dble
 xyc_s(1:2) = (/ellipse_s(i_ell_x_c), ellipse_s(i_ell_y_c)/)
 
 if (verbose >= v_debug) then
-  print '(a,2F10.6)','spot_limb_eclipse: xy_tng,1 = ',xy_tng(1:2,1)
-  print '(a,2F10.6)','                   xy_tng,2 = ',xy_tng(1:2,2)
-end if
+  print *,'spot_limb_eclipse: xy_tng,1 = ',xy_tng(1:2,1)
+  print *,'                   xy_tng,2 = ',xy_tng(1:2,2)
+endif
 
 area_flags_c = ell_ucircle_overlap(ellipse,verbose1,xy_int_c,t_c_e,t_c_c)
 area_c = area_flags_c(1)
@@ -289,21 +290,21 @@ area_s = area_flags_s(1)
 flag_s = int(area_flags_s(2))
 if (verbose >= v_debug)  then
   print *,'spot_limb_eclipse: flag_c, flag_s = ',flag_c, flag_s
-end if
+endif
 
 ! Ellipse does not overlap circle - 
 if (btest(flag_c, b_ell_no_overlap)) then
   spot_limb_eclipse = 0.0d0
   if (verbose >= v_debug) print *,'spot_limb_eclipse: 0, ',spot_limb_eclipse
   return
-end if
+endif
 
 ! Ellipse completely covers the circle
 if (btest(flag_c, b_ell_circle_inside_ellipse)) then
   spot_limb_eclipse = 1.0d0
   if (verbose >= v_debug) print *,'spot_limb_eclipse: 1, ',spot_limb_eclipse
   return
-end if
+endif
 
 
 ! Value of test_c_mid_tng is used to test on which side of the line-of-tangents
@@ -329,8 +330,8 @@ if (btest(flag_c, b_ell_ellipse_inside_circle).and. &
     spot_limb_eclipse = 0.0d0
     if (verbose >= v_debug) print *,'spot_limb_eclipse: 2, ',spot_limb_eclipse
     return
-  end if
-end if
+  endif
+endif
 
 ! If ellipse is completely inside the tangent ellipse and large is false,
 ! the results is also 0.
@@ -338,7 +339,7 @@ if ((.not.large).and.(btest(flag_s,b_ell_1_inside_2))) then
   spot_limb_eclipse = 0.0d0
   if (verbose >= v_debug) print *,'spot_limb_eclipse: 3, ',spot_limb_eclipse
   return
-end if
+endif
 
 ! Area of smaller elliptical arc for the tangent ellipse defined by the tangent 
 ! points
@@ -367,8 +368,8 @@ if (btest(flag_c, b_ell_ellipse_inside_circle).and. &
     spot_limb_eclipse = ellipse(i_ell_area)/crescent_area
     if (verbose >= v_debug) print *,'spot_limb_eclipse: 5, ',spot_limb_eclipse
     return
-  end if
-end if
+  endif
+endif
 
 ! If the occulting ellipse is completely inside the tangent ellipse. We
 ! already dealt with the case large is .false., here is the result if it is 
@@ -377,7 +378,7 @@ if (btest(flag_s,b_ell_1_inside_2)) then
   spot_limb_eclipse = ellipse(i_ell_area)/lens_area
   if (verbose >= v_debug) print *,'spot_limb_eclipse: 6, ',spot_limb_eclipse
   return
-end if
+endif
 
 ! Tangent ellipse is completely inside ellipse 1 
 if (btest(flag_s, b_ell_2_inside_1)) then
@@ -417,8 +418,8 @@ if (btest(flag_s, b_ell_2_inside_1)) then
       if (verbose >= v_debug) print *,'spot_limb_eclipse: 9',spot_limb_eclipse
       return
 
-    end if
-  end if
+    endif
+  endif
   ! Still dealing here with the case of the tangent ellipse completely inside
   ! the occulting ellipse.
   !
@@ -485,14 +486,17 @@ if (btest(flag_s, b_ell_2_inside_1)) then
     endif
 
   case default
-    print *,'spot_limb_eclipse: n_onside = ',n_onside
-    print *,ellipse
-    print *,ellipse_s
-    stop
+    if (verbose >= v_debug) then
+      print *,'spot_limb_eclipse: n_onside = ',n_onside
+      print *,ellipse
+      print *,ellipse_s
+    endif
+    spot_limb_eclipse = bad_dble
+    return
 
   end select
 
-end if
+endif
 
 ! Occulting ellipse has four intersections with the circle and two with the
 ! tangent ellipse
@@ -550,7 +554,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       k_in = 1
     else
       k_in = 2
-    end if
+    endif
     ! Calculate the area of the three-sided shape defined by the two elliptical
     ! arcs and the circular arc between by the tangent point, the ellipse-
     ! ellipse intersection point and the ellipse-circle intersection point.
@@ -594,7 +598,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         spot_limb_eclipse = 1.0d0 - (circ_arc_area - ell_arc_area)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 21', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = 1.0d0 -  &
@@ -602,7 +606,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         /crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 22', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
@@ -613,18 +617,18 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         (circ_arc_area - ell_arc_area+ellipse_s(i_ell_area)-area_s)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 23', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = 1.0d0 -  (circ_arc_area-ell_arc_area) &
         /crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 24', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
-    end if
+    endif
 
   case (3)
     ! Find which ellipse-ellipse intersection point is on the opposite side of 
@@ -642,7 +646,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       k_in = 1
     else
       k_in = 2
-    end if
+    endif
     ! Calculate the area of the three-sided shape defined by the two elliptical
     ! arcs and the circular arc between by the tangent point inside the
     ! occulting ellipse, the ellipse-ellipse intersection point on the
@@ -683,7 +687,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       ell_arc_area = ellipse(i_ell_area) &
                    - ell_sector_area(t_off(1),t_s_e(j_off),ellipse) &
                    + triangle_area(xy_off(1:2,1), xy_int_s(1:2,j_off), xyc_e)
-    end if
+    endif
 
     ! Area of elliptical arc for tangent ellipse - same test with mid-point
     xy_m = ell_t_to_xy(midangle(t_tng(k_in),t_s_s(j_off)),ellipse_s)
@@ -696,20 +700,20 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       ell_arc_area_s = ellipse_s(i_ell_area)  &
                      - ell_sector_area(t_tng(k_in),t_s_s(j_off),ellipse_s)  &
                      + triangle_area(xy_tng(1:2,k_in),xy_int_s(1:2,j_off),xyc_s)
-    end if
+    endif
     area_t = ell_arc_area + circ_arc_area - ell_arc_area_s &
            + triangle_area(xy_off(1:2,1),xy_tng(1:2,k_in),xy_int_s(1:2,j_off))
     if (large) then
       spot_limb_eclipse = (area_c - area_t)/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 25', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse = (area_c-area_s-area_t)/crescent_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 26', spot_limb_eclipse
-      end if
+      endif
       return
     endif
 
@@ -725,14 +729,14 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       - (pi - area_c - ellipse_s(i_ell_area)- area_s)/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 27', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse = 1.0d0 - (pi - area_c - ellipse_s(i_ell_area)- area_s)&
       / crescent_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 28', spot_limb_eclipse
-      end if
+      endif
       return
     endif
   else
@@ -740,19 +744,19 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       spot_limb_eclipse = area_c/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 29', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse = (area_c-area_s)/crescent_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 30', spot_limb_eclipse
-      end if
+      endif
       return
     endif
-  end if
+  endif
 
   end select
-end if
+endif
 
 ! Occulting ellipse intersects circle at two points but has no overlap with
 ! tangent  ellipse.
@@ -774,16 +778,16 @@ if (btest(flag_c, b_ell_two_intersects).and. &
     spot_limb_eclipse = area_c/lens_area
     if (verbose >= v_debug) then
       print *,'spot_limb_eclipse: 32', spot_limb_eclipse
-    end if
+    endif
     return
   else
     spot_limb_eclipse = area_c/crescent_area
     if (verbose >= v_debug) then
       print *,'spot_limb_eclipse: 33', spot_limb_eclipse
-    end if
+    endif
     return
   endif
-end if
+endif
 
 
 ! Occulting ellipse intersects circle at two points and tangent ellipse at two
@@ -811,13 +815,24 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         spot_limb_eclipse = area_c/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 34', spot_limb_eclipse
-        end if
+         !!do i_on=1,2
+         !!  write (34,*) xy_int_s(1:2,i_on)
+         !!end do
+         !!write (35,*) xy_tng(1:2,1)
+         !!write (35,*) xy_tng(1:2,2)
+         !!do i_on=1,10001
+         !!  t_m = 2*pi*i_on/10001.0d0
+         !!  write (36,*) cos(t_m),sin(t_m),&
+         !!    ell_t_to_xy(t_m,ellipse_s), &
+         !!    ell_t_to_xy(t_m,ellipse)
+         !!end do
+        endif
         return
       else
         spot_limb_eclipse = (area_c-area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 35', spot_limb_eclipse
-        end if
+        endif
         return
       endif
     else
@@ -826,19 +841,19 @@ if (btest(flag_c, b_ell_two_intersects).and. &
                           / lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 36', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = 1.0d0 - &
            (area_c - area_s) / crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 37', spot_limb_eclipse
-        end if
+        endif
         return
       endif
-    end if
+    endif
 
-  end if
+  endif
 
   ! Both ellipse-ellipse intersection points on the opposite side to mid-point
   ! on circle between ellipse-circle tangent point
@@ -851,13 +866,13 @@ if (btest(flag_c, b_ell_two_intersects).and. &
                           / lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 38', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = 1.0d0
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 39', spot_limb_eclipse
-        end if
+        endif
         return
       endif
     else
@@ -865,18 +880,18 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         spot_limb_eclipse = area_s/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 40', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = 0.0d0
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 41', spot_limb_eclipse
-        end if
+        endif
         return
       endif
     endif
 
-  end if
+  endif
 
   ! Remaining case for two intersections of occulting ellipse with circle and
   ! tangent ellipse is ellipse-ellipse intersections on opposite side of the
@@ -906,7 +921,7 @@ if (btest(flag_c, b_ell_two_intersects).and. &
     k_in = 1
   else
     k_in = 2
-  end if
+  endif
 
   circ_arc_area =  arc_area(xy_int_c(1:2,i_on),xy_tng(1:2,k_in))
   ell_arc_area = ell_sector_area(t_c_e(i_on),t_s_e(j_on),ellipse) - &
@@ -926,9 +941,9 @@ if (btest(flag_c, b_ell_two_intersects).and. &
     return
   endif
 
-end if
+endif
 
-! Occuting ellipse intersects tangent ellipse at two points and is completely
+! Occulting ellipse intersects tangent ellipse at two points and is completely
 ! inside the circle
 if (btest(flag_c, b_ell_ellipse_inside_circle).and. &
     btest(flag_s, b_ell_two_intersects) ) then
@@ -957,7 +972,7 @@ if (btest(flag_c, b_ell_ellipse_inside_circle).and. &
       return
     endif
   endif
-end if
+endif
 
 ! Occulting ellipse intersects the tangent ellipse at four points and is
 ! completely inside circle
@@ -1028,7 +1043,7 @@ if (btest(flag_c, b_ell_ellipse_inside_circle).and. &
 
   end select
 
-end if
+endif
 
 ! Occulting ellipse intersects the tangent ellipse at four points and the
 ! circle at two points.
@@ -1072,14 +1087,14 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         spot_limb_eclipse = 1.0d0 - (pi - area_c)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 55', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  1.0d0 - &
           (pi-area_c-ellipse_s(i_ell_area)+area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 56', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
@@ -1089,17 +1104,17 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         spot_limb_eclipse = area_c/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 57', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  (area_c-area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 58', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
-    end if
+    endif
 
   case (1)
     ! Which ellipse-circle intersection point is "off-side"?
@@ -1116,7 +1131,7 @@ if (btest(flag_c, b_ell_two_intersects).and. &
       k_in = 1
     else
       k_in = 2
-    end if
+    endif
 
     ! Find the mid-point on the circle between xy_int_c(1:2,i_off) and 
     ! xy_tng(1:2,k_in) then check whether this is inside or outside the
@@ -1130,7 +1145,7 @@ if (btest(flag_c, b_ell_two_intersects).and. &
     else
       circ_arc_area =  pi - arc_area(xy_int_c(1:2,i_off),xy_tng(1:2,k_in))
       lcswitch = .true.
-    end if
+    endif
     ! Area of elliptical arc for occulting ellipse depends on whether mid-point
     ! between xy_int_c(1:2,i_off) and xy_off(1:2,1) is on the same side of the
     ! line between these points as the tangent point inside the occulting
@@ -1151,8 +1166,8 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         ell_arc_area = ellipse(i_ell_area)   &
           -  ell_sector_area(t_off_e(1), t_c_e(i_off) ,ellipse) &
           + triangle_area(xy_off(1:2,1),xy_int_c(1:2,i_off),xyc_e)
-      end if
-    end if
+      endif
+    endif
     ! Area of elliptical arc for tangent ellipse depends on whether the
     ! mid-point between xy_tng(1:2,k_in) and xy_off(1:2,1) is on the same side
     ! or opposite side of the line between these points as xy_m_c
@@ -1175,13 +1190,13 @@ if (btest(flag_c, b_ell_two_intersects).and. &
       spot_limb_eclipse = (area_c-area_t)/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 59', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse =  (area_c-area_t-area_s)/crescent_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 60', spot_limb_eclipse
-      end if
+      endif
       return
     endif
     
@@ -1219,14 +1234,14 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         spot_limb_eclipse = (area_c - ell_arc_area + ell_arc_area_s)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 61', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = (area_c - ell_arc_area + ell_arc_area_s - area_s) &
                             / crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 62', spot_limb_eclipse
-        end if
+        endif
         return
       endif
     else
@@ -1254,17 +1269,17 @@ if (btest(flag_c, b_ell_two_intersects).and. &
         spot_limb_eclipse = (area_s + ell_arc_area - ell_arc_area_s)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 63', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse = (ell_arc_area- ell_arc_area_s) / crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 64', spot_limb_eclipse
-        end if
+        endif
         return
       endif
        
-    end if
+    endif
 
   case (3)
     ! Which ellipse-circle intersection point is "on-side"?
@@ -1281,7 +1296,7 @@ if (btest(flag_c, b_ell_two_intersects).and. &
       k_in = 1
     else
       k_in = 2
-    end if
+    endif
 
     circ_arc_area =  arc_area(xy_int_c(1:2,i_on),xy_tng(1:2,k_in))
     ell_arc_area = ell_sector_area(t_c_e(i_on),t_on_e(1),ellipse) - &
@@ -1306,19 +1321,19 @@ if (btest(flag_c, b_ell_two_intersects).and. &
       spot_limb_eclipse = area_s/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 67', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse =  0.0d0
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 68 ', spot_limb_eclipse
-      end if
+      endif
       return
     endif
 
   end select
 
-end if 
+endif 
 
 ! Occulting ellipse intersects the circle at four points but does not overlap
 ! the tangent ellipse.
@@ -1333,21 +1348,21 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       spot_limb_eclipse = area_c/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 69', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse =  area_c/crescent_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 70 ', spot_limb_eclipse
-      end if
+      endif
       return
     endif
   else 
     spot_limb_eclipse =  0.0d0
     if (verbose >= v_debug) print *,'spot_limb_eclipse: 71 ', spot_limb_eclipse
-  end if
+  endif
 
-end if
+endif
 
 if (btest(flag_c, b_ell_four_intersects).and. &
     btest(flag_s, b_ell_four_intersects) ) then
@@ -1394,7 +1409,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         xy_off_c(1:2,n_off_c) = xy_int_c(1:2,i)
         t_off_c_e(n_off_c) = t_c_e(i)
         t_off_c_c(n_off_c) = t_c_c(i)
-      end if
+      endif
     end do
 
     select case (n_on_c)
@@ -1415,7 +1430,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           circ_arc_area = pi - arc_area(xy_off_c(1:2,1), xy_off_c(1:2,2))
         else 
           circ_arc_area = arc_area(xy_off_c(1:2,1), xy_off_c(1:2,2))
-        end if
+        endif
 
         ell_arc_area = ell_sector_area(t_off_c_e(1),t_off_c_e(2),ellipse) - &
                  triangle_area(xy_off_c(1:2,1), xy_off_c(1:2,2), xyc_e)
@@ -1424,28 +1439,28 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           / lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 72 ', spot_limb_eclipse
-        end if
+        endif
         return
         
       else
         spot_limb_eclipse =  (ellipse_s(i_ell_area) - area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 73 ', spot_limb_eclipse
-        end if
-      end if
+        endif
+      endif
 
       case (4)
       if (large) then
         spot_limb_eclipse = area_s/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 74', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  (area_c-area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 75 ', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
@@ -1470,7 +1485,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         xy_off_c(1:2,n_off_c) = xy_int_c(1:2,i)
         t_off_c_e(n_off_c) = t_c_e(i)
         t_off_c_c(n_off_c) = t_c_c(i)
-      end if
+      endif
     end do
 
     ! Which tangent point is inside the occulting ellipse?
@@ -1478,7 +1493,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       k_in = 1
     else
       k_in = 2
-    end if
+    endif
 
     select case (n_on_c)
 
@@ -1523,13 +1538,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         spot_limb_eclipse = (ellipse_s(i_ell_area)+area_t+area_e)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 76', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  (area_t+area_e)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 77 ', spot_limb_eclipse
-        end if
+        endif
         return
       endif
       
@@ -1550,7 +1565,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         circ_arc_area = pi - arc_area(xy_off_c(1:2,1), xy_tng(1:2,k_in))
       else 
         circ_arc_area = arc_area(xy_off_c(1:2,1), xy_tng(1:2,k_in))
-      end if
+      endif
 
       ! Area of elliptical arc for occulting ellipse
       ell_arc_area = ell_sector_area(t_off_c_e(1),t_off_e(1),ellipse) - &
@@ -1580,13 +1595,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         spot_limb_eclipse = (area_c - area_t)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 78', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  (area_c-area_t - area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 79 ', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
@@ -1609,7 +1624,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         t_on_c_e(n_on_c) = t_c_e(i)
         t_on_c_c(n_on_c) = t_c_c(i)
         if (n_on_c == 2) exit
-      end if
+      endif
     end do
     if (ell_point_is_inside(xy_tng(1:2,1),ellipse)) then
         ! Calculate the sum of the areas of the two three-sided shapes defined
@@ -1624,7 +1639,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         else
           i_c_1 = 2
           i_c_2 = 1
-        end if
+        endif
         
         if (distance(xy_tng(1:2,1), xy_on(1:2,1)) < & 
             distance(xy_tng(1:2,1), xy_on(1:2,2)) ) then
@@ -1633,7 +1648,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         else
           i_s_1 = 2
           i_s_2 = 1
-        end if
+        endif
         ! First tangent point
         circ_arc_area = arc_area(xy_on_c(1:2,i_c_1), xy_tng(1:2,1))
         ell_arc_area = &
@@ -1658,13 +1673,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           spot_limb_eclipse = (area_s +  area_t)/lens_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 80', spot_limb_eclipse
-          end if
+          endif
           return
         else
           spot_limb_eclipse =  area_t/crescent_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 81', spot_limb_eclipse
-          end if
+          endif
           return
         endif
 
@@ -1687,7 +1702,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           xy_off_c(1:2,n_off_c) = xy_int_c(1:2,i)
           t_off_c_e(n_off_c) = t_c_e(i)
           t_off_c_c(n_off_c) = t_c_c(i)
-        end if
+        endif
       end do
       select case (n_on_c)
       case (0) 
@@ -1716,13 +1731,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           spot_limb_eclipse = (area_s - ell_arc_area - ell_arc_area_s)/lens_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 82', spot_limb_eclipse
-          end if
+          endif
           return
         else
           spot_limb_eclipse =  (ell_arc_area - ell_arc_area_s)/crescent_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 83 ', spot_limb_eclipse
-          end if
+          endif
           return
         endif
 
@@ -1739,7 +1754,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         else
           i_c_1 = 2
           i_c_2 = 1
-        end if
+        endif
         
         if (distance(xy_tng(1:2,1), xy_on(1:2,1)) < & 
             distance(xy_tng(1:2,1), xy_on(1:2,2)) ) then
@@ -1748,7 +1763,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         else
           i_s_1 = 2
           i_s_2 = 1
-        end if
+        endif
         ! First tangent point
         circ_arc_area = arc_area(xy_on_c(1:2,i_c_1), xy_tng(1:2,1))
         ell_arc_area = &
@@ -1773,13 +1788,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           spot_limb_eclipse = (area_s + crescent_area - area_t)/lens_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 84', spot_limb_eclipse
-          end if
+          endif
           return
         else
           spot_limb_eclipse =  1.0d0 - area_t/crescent_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 85 ', spot_limb_eclipse
-          end if
+          endif
           return
         endif
 
@@ -1809,20 +1824,20 @@ if (btest(flag_c, b_ell_four_intersects).and. &
           spot_limb_eclipse = (area_c - ell_arc_area + ell_arc_area_s)/lens_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 86', spot_limb_eclipse
-          end if
+          endif
           return
         else
           spot_limb_eclipse = (area_c - ell_arc_area + ell_arc_area_s - area_s)&
             /crescent_area
           if (verbose >= v_debug) then
             print *,'spot_limb_eclipse: 87 ', spot_limb_eclipse
-          end if
+          endif
           return
         endif
 
       end select ! n_on_c
 
-    end if
+    endif
 
   case (3) !  = n_off_s, number of off-side ellipse-ellipse intersections
     ! Still in 4 ellipse-ellipse and 4 ellipse-circle intersections case
@@ -1846,7 +1861,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         xy_off_c(1:2,n_off_c) = xy_int_c(1:2,i)
         t_off_c_e(n_off_c) = t_c_e(i)
         t_off_c_c(n_off_c) = t_c_c(i)
-      end if
+      endif
     end do
 
     ! Which tangent point is inside the occulting ellipse?
@@ -1854,7 +1869,7 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       k_in = 1
     else
       k_in = 2
-    end if
+    endif
 
     select case (n_off_c)
 
@@ -1897,13 +1912,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         spot_limb_eclipse = (area_c - area_t -area_e)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 88', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  (area_c - area_t -area_e-area_s)/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 89 ', spot_limb_eclipse
-        end if
+        endif
         return
       endif
       
@@ -1926,13 +1941,13 @@ if (btest(flag_c, b_ell_four_intersects).and. &
         spot_limb_eclipse = (area_s + area_t)/lens_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 90', spot_limb_eclipse
-        end if
+        endif
         return
       else
         spot_limb_eclipse =  area_t/crescent_area
         if (verbose >= v_debug) then
           print *,'spot_limb_eclipse: 91 ', spot_limb_eclipse
-        end if
+        endif
         return
       endif
 
@@ -1945,18 +1960,18 @@ if (btest(flag_c, b_ell_four_intersects).and. &
       spot_limb_eclipse = area_s/lens_area
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 92', spot_limb_eclipse
-      end if
+      endif
       return
     else
       spot_limb_eclipse = 0.0d0
       if (verbose >= v_debug) then
         print *,'spot_limb_eclipse: 93 ', spot_limb_eclipse
-      end if
+      endif
       return
     endif
   end select
 
-  end if
+  endif
 
 end function spot_limb_eclipse
 
@@ -1965,8 +1980,8 @@ end function spot_limb_eclipse
 subroutine project_spot(alpha_s, beta_s, gamma_s, ellipse_s, uv_tng)
 !
 ! Ellipse that is the projection of a circular spot on a spherical star onto
-! the plane of the sky, as the coordinates of the tangent points of this
-! ellipse with the projected limb of the star for stars on the limb.
+! the plane of the sky, and the coordinates of the tangent points of this
+! ellipse with the projected limb of the star for spots on the limb.
 !
 ! The spot is assumed to be on a spherical star of unit radius centred on the
 ! origin. The spot edge is defined by the intersection of a cone with its vertex
@@ -2018,8 +2033,8 @@ else
     uv_tng(2,2) =  a_p*cosa*cost_t  - b_p*sina*sint_t + v_c
   else
     uv_tng(:,:) = not_set_dble
-  end if
-end if
+  endif
+endif
 
 
 end subroutine project_spot
