@@ -19,6 +19,19 @@ module stellar
 !
 ! HISTORY
 ! -------
+!  24 Feb 2017
+!   Added catch to avoid getting stuck in while loop.
+!
+! 5 May 2017
+!  Added tabulated limb darkening law.
+!
+! 14 Sep 2016
+! Improved definition/handling of coordinates to fix problems with radial
+! velocities in bright().
+! Improved calculation of local surface gravity - now done at Roche
+! equipotential instead of at the ellipsoid surface.
+! p.maxted@keele.ac.uk
+!
 ! Apr 2016
 ! Added exact_grav option to bright
 ! p.maxted@keele.ac.uk
@@ -51,9 +64,9 @@ public delta_func    ! Helper function for t_ecl_to_peri
 
 contains
 
-function starshape(radius, rsep, frot, ecc, qmass, model, rtol, &
+function starshape(radius, rsep, spar, ecc, q, model, rtol, &
                    verbose) result(abcd)
-implicit none
+implicit none 
 ! Semi-major axes (A,B,C) and offset towards companion, D, from centre-of-mass
 ! to geometrical centre of an ellipsoid that approximates the shape of a
 ! star in a binary system. 
@@ -81,14 +94,18 @@ implicit none
 !         calculated. For the Roche model only, set radius=1 to specify
 !         a star that fills its Roche lobe.
 ! RSEP  = Separation of the stars in units of the semi-major axis.
-! FROT  = angular velocity of star/pseudo-synchronous angular velocity
+! SPAR  = angular velocity of star/pseudo-synchronous angular velocity OR
+!         for Correia (2014A&A...570L...5C) model, fluid second Love number 
+!         for radial displacement, h_f.
 ! ECC   = eccentricity
-! QMASS = mass of companion/mass of star
+! Q     = mass of companion/mass of star
 ! RTOL  = Tolerance on fractional difference (ABC**(1/3)-radius)/radius.
 !         If RTOL <= EPSILON(0.0), e.g., 0.0, then EPSILON(0.0) 
 !         (machine precision) is used as the tolerance.
 !
-! MODEL = -1 - Roche model
+! MODEL
+!       = -2 - Roche model with precise volume calculation
+!       = -1 - Roche model
 !       =  0 - spherical star
 !       =  1 - polytrope n = 1.5
 !       =  2 - polytrope n = 3
@@ -103,33 +120,38 @@ implicit none
 
 double precision, intent(in) :: radius  ! Radius of a sphere with the same volume as the ellipsoid
 double precision, intent(in) :: rsep    ! Separation of the stars in units of the semi-major axis
-double precision, intent(in) :: frot    ! angular velocity of star/pseudo-synchronous angular velocity
+double precision, intent(in) :: spar    ! angular velocity of star/pseudo-synchronous angular velocity
 double precision, intent(in) :: ecc     ! eccentricity of the orbit
-double precision, intent(in) :: qmass   ! mass of companion/mass of star
+double precision, intent(in) :: q       ! mass of companion/mass of star
 double precision, intent(in) :: rtol    ! Tolerance on fractional difference (ABC**(1/3)-radius)/radius
 integer, intent(in)  :: model   ! Model used to calculate star shape (see constants)
 integer, intent(in)  :: verbose ! Verbosity level for error reporting
 double precision             :: abcd(4) ! semi-major axis on x-axis towards companion
 
 ! Local variables
-double precision ::  rt, t, ttol, tlo, thi
-double precision ::  a,b,c,d
-double precision ::  rsep_l
+double precision :: rt, t, ttol, tlo, thi
+double precision :: a,b,c,d
+double precision :: ksi,r_0,vfac,n,n2,r2,r3,r4,r5,r6,q2,q3
+double precision :: rsep_l
+integer :: it
+integer, parameter :: itmax = 100
+
+abcd = (/bad_dble, bad_dble, bad_dble, bad_dble/)
+a = not_set_dble
+b = not_set_dble
+c = not_set_dble
+d = not_set_dble
 
 if(verbose >= v_debug) then
   print *,'starshape: radius, q, ecc, rsep, model = ', &
-  radius,qmass,ecc,rsep,model
-end if 
+  radius,q,ecc,rsep,model
+endif 
 if ((radius <= 0.d0).or.(radius.gt.1.d0))then
-  a = bad_dble
-  b = bad_dble
-  c = bad_dble
-  d = bad_dble
   if(verbose >= v_error)then
     print *,'starshape: error: invalid input radius: ',radius
-  end if
+  endif
   return
-end if
+endif
 
 ! If rsep is within 4*epsilon(0.d0) of 1-ecc or 1+ecc then assume that this
 ! is due to round-off error 
@@ -144,13 +166,9 @@ else if (abs(tlo) < ttol) then
 else if (abs(thi) < ttol) then
   rsep_l = (1.d0+ecc)
 else
-  a = bad_dble
-  b = bad_dble
-  c = bad_dble
-  d = bad_dble
   if(verbose >= v_error)then
     print *,'starshape: error: rsep out of range: ', rsep,ecc
-  end if
+  endif
   return
 endif
 
@@ -159,19 +177,68 @@ rt = radius
 
 select case (model)
 
-case(starshape_roche)
+case(starshape_roche_v)
 
-  t = 1.d0
+  t = 1
+  it = 0 
+  d = 0
   do while (t.gt.ttol)
-    call shape_roche(rt,rsep_l,frot,qmass,a,b,c,d,verbose-1)
+    it = it + 1
+    if (it > itmax) then
+      if(verbose >= v_error) then
+        print *,'starshape: failed to converge : t  = ',t
+      endif
+      return
+    endif
+
     if(d.eq.bad_dble) return
     if (radius.eq.1.d0) then
-      t = 0.d0
+      t = 0
+    else
+      ksi = rsep_l*roche(x=rt,y=0.0d0,z=0.0d0,d=rsep_l,q=q,f=1.0d0)
+      n = (q+1d0)/2d0
+      r_0 = 1d0/(ksi-q)
+      r2 = r_0**2
+      r3 = r2*r_0
+      r4 = r3*r_0
+      r5 = r4*r_0
+      r6 = r5*r_0
+      q2 = q**2
+      q3 = q2*q
+      n2 = n**2
+      vfac = (1d0 + r6*(12d0*q2/5d0  &
+      + 15d0*q2*r2/7d0 + 2d0*q2*r4 + 22d0*q3*r3/7d0 + 157*q3*r5/7d0 &
+      + 2d0*n/r3 + 32d0*n2/5d0 + 176*n**3*r3/7d0 + 8d0*n*q/5d0 &
+      + 296*n*q*(2d0*q+n)*r3/35d0 + 26d0*n*q*(q+3d0*n)*r5/35d0 ))
+      t = vfac**third*r_0*rsep_l/radius
+      rt = rt/t
+      t = abs(t-1.d0)
+    endif
+  end do
+  call shape_roche(rt,rsep_l,spar,q,a,b,c,d,verbose-1)
+
+case(starshape_roche)
+
+  t = 1
+  it = 0
+  do while (t.gt.ttol)
+    it = it + 1
+    if (it > itmax) then
+      if(verbose >= v_error) then
+        print *,'starshape: failed to converge : t  = ',t
+      endif
+      return
+    endif
+
+    call shape_roche(rt,rsep_l,spar,q,a,b,c,d,verbose-1)
+    if(d.eq.bad_dble) return
+    if (radius.eq.1.d0) then
+      t = 0
     else
       t = (a*b*c)**third/radius
       rt = rt/t
       t = abs(t-1.d0)
-    end if
+    endif
   end do
 
 case(starshape_sphere)
@@ -179,14 +246,22 @@ case(starshape_sphere)
   a = radius
   b = radius
   c = radius
-  d = 0.d0
-  t = 0.d0
+  d = 0
+  t = 0
 
 case(starshape_poly1p5)
 
-  t = 1.d0
+  t = 1
+  it = 0
   do while (t.gt.ttol)
-    call shape_n1p5(rt,rsep_l,frot,ecc,qmass,a,b,c,d,verbose-1)
+    it = it + 1
+    if (it > itmax) then
+      if(verbose >= v_error) then
+        print *,'starshape: failed to converge : t, radius  = ',t,radius
+      endif
+      return
+    endif
+    call shape_n1p5(rt,rsep_l,spar,ecc,q,a,b,c,d,verbose-1)
     if(d.eq.bad_dble) return
     t = (a*b*c)**third/radius
     rt = rt/t
@@ -195,9 +270,37 @@ case(starshape_poly1p5)
 
 case(starshape_poly3p0)
 
-  t = 1.d0
+  t = 1
+  it = 0
   do while (t.gt.ttol)
-    call shape_n3p0(rt,rsep_l,frot,ecc,qmass,a,b,c,d,verbose-1)
+    it = it + 1
+    if (it > itmax) then
+      if(verbose >= v_error) then
+        print *,'starshape: failed to converge : t  = ',t
+      endif
+      return
+    endif
+    call shape_n3p0(rt,rsep_l,spar,ecc,q,a,b,c,d,verbose-1)
+    if(d.eq.bad_dble) return
+    t = (a*b*c)**third/radius
+    rt = rt/t
+    t = abs(t-1.d0)
+  end do
+
+
+case(starshape_love)
+
+  t = 1
+  it = 0
+  do while (t.gt.ttol)
+    it = it + 1
+    if (it > itmax) then
+      if(verbose >= v_error) then
+        print *,'starshape: failed to converge : t  = ',t
+      endif
+      return
+    endif
+    call shape_love(rt,rsep_l,spar,q,a,b,c,d,verbose-1)
     if(d.eq.bad_dble) return
     t = (a*b*c)**third/radius
     rt = rt/t
@@ -205,18 +308,16 @@ case(starshape_poly3p0)
   end do
 
 case default
-  a = bad_dble
-  b = bad_dble
-  c = bad_dble
-  d = bad_dble
   if(verbose >= v_error) then
     print *,'starshape: error: invalid input model'
-  end if
+  endif
   return
 
 end select
 
-if(verbose >= 4) print *,'starshape: normal exit : a,b,c,d,t  = ',a,b,c,d,t
+if(verbose >= v_debug) then
+  print *,'starshape: normal exit : a,b,c,d,t  = ',a,b,c,d,t
+endif
 
 abcd = (/a, b, c, d/)
 return
@@ -274,7 +375,7 @@ if (ar < 0.d0) then
   d = bad_dble
   if(verbose >= v_error) print *,'shape_n1p5: radius out of range.',radius,frot,ecc,qmass
   return
-end if
+endif
 
 ! Interpolate values of fractional changes in radius
 if(ar < 1.0d-2) then
@@ -300,7 +401,7 @@ else if(ar <= 1.09d-2) then
 else
   print *,'shape_n1p5: invalid value of ar',ar
   stop
-end if
+endif
 
 a = radius*(1.d0+dxie)
 b = a
@@ -369,7 +470,7 @@ else if(a <= 1.09d-2) then
   func_n1p5 = a-par(1)*((1.d0-w)*array(24) + w*array(25))
 else
   func_n1p5 = bad_dble
-end if
+endif
 return
 
 end function func_n1p5
@@ -424,7 +525,7 @@ if (ar < 0.d0) then
   d = bad_dble
   if(verbose >= 1) print *,'shape_n3p0: radius out of range.',radius,frot,ecc,qmass
   return
-end if
+endif
 
 ! Interpolate values of fractional changes in radius
 if(ar.lt.9.0d-4) then
@@ -448,7 +549,7 @@ else if(ar <= 9.7d-4) then
 else
   print *,'shape_n3p0: invalid value of ar',ar
   stop
-end if
+endif
 a = radius*(1.d0+dxie)
 b = a
 c = radius*(1.d0+dxip)
@@ -514,7 +615,7 @@ else if(a <=  9.7d-4) then
   func_n3p0 = a-par(1)*(w*array(ii) + (1.d0-w)*array(ii+1))
 else
   func_n3p0 = bad_dble
-end if
+endif
 return
 end function func_n3p0
 
@@ -528,43 +629,83 @@ double precision, intent(out) :: a, b, c, d
 integer, intent(in)           ::  verbose
 
 ! Local variables
-double precision   :: pot, a1,  rl1
+double precision   :: pot, rl1, xf, xb
 integer, parameter :: npar = 4
 double precision   :: par(npar)
-double precision, parameter :: tol=1.0d-9
+double precision, parameter :: tol=1.0d-6
 
 rl1 = roche_l1(qmass,frot)
 if (radius == 1.d0) then
-  a = rl1
+  xf = rl1
 else
   if (radius > rl1) then
     if(verbose >= v_error) then
       print *,' shape_roche: star exceeds roche lobe'
       print *,' shape_roche: radius,rl1,q = ',radius,rl1,qmass
-    end if
+    endif
     a = bad_dble
     b = bad_dble
     c = bad_dble
     d = bad_dble
     return
-  end if
-  a = radius
-end if
-pot = roche(a,0.d0,0.d0,rsep,qmass,frot)
+  endif
+  xf = radius
+endif
+pot = roche(xf,0.d0,0.d0,rsep,qmass,frot)
 par(1) = pot
 par(2) = rsep
 par(3) = qmass
 par(4) = frot
-b = zbrent(func_b,a*0.5d0,a*1.1d0,tol,npar,par,verbose-1)
-c = zbrent(func_c,a*0.5d0,a*1.1d0,tol,npar,par,verbose-1)
-a1 = zbrent(func_a,-a*1.5d0,-a*0.5d0,tol,npar,par,verbose-1)
-d = 0.5*(a1+a)
-a = a - d
+b = zbrent(func_b,radius*0.5d0,radius*1.1d0,tol,npar,par,verbose-1)
+c = zbrent(func_c,radius*0.5d0,radius*1.1d0,tol,npar,par,verbose-1)
+xb = zbrent(func_a,-radius*1.5d0,-radius*0.5d0,tol,npar,par,verbose-1)
+d = 0.5*(xf+xb)
+a = xf - d
 
 end subroutine shape_roche
 
 !-----------------------------------------------------------------------
 
+subroutine shape_love(radius,rsep,h_f,qmass,a,b,c,d,verbose)
+implicit none
+
+double precision, intent(in)  :: radius, rsep, h_f, qmass
+double precision, intent(out) :: a, b, c, d
+integer, intent(in)           :: verbose
+double precision              :: qr
+
+! RADIUS = Radius of a spherical planet in unit of stellar radii.
+! RSEP   = Separation of between bodies in unit of stellar radii.
+! qmass  = mass of star/mass of planet
+! h_f    = fluid second love number as used in Correia()
+! a,b,c  = the semi-major axes of the ellipse in units of stellar radii
+! d      = d is an offset towards the companion from the centre-of-mass of the
+!           planet to the geometric centre of the ellipsoid
+!
+! Local variables.
+!qr      = planet asymmetry as defined in correia
+
+
+b=radius
+qr=0.5d0*h_f*qmass*(b/rsep)**3       !from eqn(12) in correia(2014)
+if (qr >= 1) then
+    if (verbose > v_silent) print *,'shape_love: qr >1; b, qr  =', b, qr
+    a = bad_dble
+    b = bad_dble
+    c = bad_dble
+    d = bad_dble
+    return
+endif
+
+a = b*(1.d0+3.d0*qr)          !eqn(10)
+c = b*(1.d0-qr)               !eqn(11)
+d = radius*qmass*radius**4    !From Chandrasekhar using delta_3 = 1
+
+if (verbose > v_debug) print *,'shape_love: a,c,d,qr =', a,c,d,qr
+end subroutine shape_love
+
+!-----------------------------------------------------------------------
+
 double precision function func_a(a,npar,par,verbose)
 implicit none
 integer, intent(in)          :: npar, verbose
@@ -598,6 +739,19 @@ end function func_c
 
 !-----------------------------------------------------------------------
 
+double precision function func_r(r,npar,par,verbose)
+implicit none
+integer, intent(in)          :: npar, verbose
+double precision, intent(in) :: r, par(npar)
+if (verbose > v_debug) print *,'func_r: ', r
+func_r=par(1)-roche(par(2)*r,par(3)*r,par(4)*r,par(5),par(6),par(7))
+!print *,'func_r (',real(r),') = ',real(func_r),', target =',real(par(1))
+!print *,'func_r',real(r),real(par(1)-func_r),real(par(1))
+return 
+end function func_r
+
+!-----------------------------------------------------------------------
+
 double precision function roche(x, y, z, d, q, f, iscomp)
 !+
 !
@@ -645,24 +799,24 @@ if (present(iscomp)) then
   iscomp_l = iscomp
 else
   iscomp_l = .false.
-end if
+endif
 
 if (q <= 0.d0) then
   roche = bad_dble
   return
-end if
+endif
 
 if (iscomp_l) then
   qq = 1.d0/q
 else
   qq = q
-end if
+endif
 
 r2 = x**2 + y**2 + z**2
 if (r2 == 0.d0) then
   roche = bad_dble
   return
-end if
+endif
 
 r = sqrt(r2)
 nu = z/r
@@ -697,30 +851,50 @@ double precision, intent(in),optional ::  f   ! Rotation factor
 !
 !-
 double complex   ::  az(6) = (/ &
-   dcmplx(-1.d0,0.0d0),dcmplx(2.0d0,0.0d0),dcmplx(-1.d0,0.0d0), &
-   dcmplx(0.d0,0.0d0),dcmplx(0.0d0,0.0d0),dcmplx(0.d0,0.0d0) /)
-double complex   ::  z
-integer ::  m
+   dcmplx(-1.d0, 0.0d0), dcmplx(2.0d0, 0.0d0), dcmplx(-1.d0, 0.0d0), &
+   dcmplx(0.d0, 0.0d0), dcmplx(0.0d0, 0.0d0), dcmplx(0.d0, 0.0d0) /)
+double complex   ::  z, tz
+integer ::  i, m
 double precision :: fac
 
 if(q <= 0.d0) then
   roche_l1 = bad_dble
   return
-end if
+endif
 
 if (present(f)) then 
   fac = f**2*(1.0d0+q)
-  az(4) =dcmplx( fac + 2.0d0*q,0.0d0)
-  az(5) =dcmplx(-2.0d0*fac - q,0.0d0)
-  az(6) =dcmplx( fac          ,0.0d0)
+  az(4) =dcmplx( fac + 2.0d0*q, 0.0d0)
+  az(5) =dcmplx(-2.0d0*fac - q, 0.0d0)
+  az(6) =dcmplx( fac          , 0.0d0)
 else
-  az(4) =dcmplx( 1.0d0 + 3.0d0*q,0.0d0)
-  az(5) =dcmplx(-2.0d0 - 3.0d0*q,0.0d0)
-  az(6) =dcmplx( 1.0d0 + q      ,0.0d0)
-end if
-z     =dcmplx( 1.0d0/(1.0d0+q),0.0d0)
-m=5
-roche_l1 = dble(laguerre(az,m,z))
+  az(4) =dcmplx( 1.0d0 + 3.0d0*q, 0.0d0)
+  az(5) =dcmplx(-2.0d0 - 3.0d0*q, 0.0d0)
+  az(6) =dcmplx( 1.0d0 + q      , 0.0d0)
+endif
+
+tz = dcmplx( 0.5d0, 0.0d0)
+m = 5
+z = laguerre(az, m, tz)
+
+if (dimag(z).eq.0.0d0) then
+  roche_l1 = dble(z)
+  return
+endif
+
+! Catch rare cases where the algorithm above converges on an imaginary root
+! Use a grid search to find a real root
+do i = 1, 49
+  tz = dcmplx(0.5d0 + 0.01d0*i - 0.02d0*i*mod(i,2), 0.0d0)
+  z = laguerre(az, m, tz)
+  if (dimag(z).eq.0.0d0) then
+    roche_l1 = dble(z)
+    return
+  endif
+enddo
+
+print *, f, q
+roche_l1 = bad_dble
 return
 
 end function roche_l1
@@ -760,20 +934,20 @@ double precision t1, t2, f21q
 if (d <= 0.d0) then
   droche = bad_dble
   return
-end if
+endif
 
 t1 = x**2 + y**2 + z**2
 if (t1 == 0.d0) then
   droche = bad_dble
   return
-end if
+endif
 t1 = t1**(-1.5d0)
 
 t2 = (d-x)**2 + y**2 + z**2
 if (t2 == 0.d0) then
   droche = bad_dble
   return
-end if
+endif
 t2 = q*t2**(-1.5d0)
 
 f21q = f**2*(1.d0+q)
@@ -791,8 +965,28 @@ double precision function limbdark(mu, law, ldc)
 ! Limb darkening
 implicit none
 double precision,intent(in) :: mu     ! Cosine of the viewing angle
-double precision,intent(in) :: ldc(:) ! Limb darkening coefficients
+double precision,intent(in) :: ldc(:) ! Limb darkening coefficients/data
 integer, intent(in) :: law    ! Limb darkening law - see constants.f90
+! If law < -1 then ldc(:) is assumed to be a grid of specific intensity values
+! tabulated on a uniform grid of abs(law) mu values, for 0 to 1. The specific
+! intensity for the input value of mu is then determined  by linear 
+! interpolation.
+
+integer :: i, n
+double precision :: w
+
+if (law  < -1 ) then
+  n =  abs(law)
+  w = 1 + mu * (n-1)
+  i = int(w)
+  w = w - dble(i)
+  if (i < n) then 
+    limbdark =  (1.0d0-w)*ldc(i) + w*ldc(i+1)
+  else
+    limbdark = ldc(n)
+  endif
+  return
+endif
 
 select case (law)
 
@@ -829,6 +1023,11 @@ case(ld_square_root) ! square-root
 case(ld_exponential) ! exponential
   limbdark = 1.d0 - ldc(1)*(1.d0-mu) - ldc(2)/(1.d0-exp(mu))
   return
+
+case(ld_power2) ! power-2 
+  limbdark = 1.d0 - ldc(1)*(1.d0-mu**ldc(2))
+  return
+
 case default ! 
   limbdark = bad_dble
 end select
@@ -888,8 +1087,8 @@ double precision,parameter  :: rtol = 0.0d0
 double precision  :: gpole,gside,gback,gfront
 double precision  :: ax(3),xarr(3),garr(3)
 
-gpole = droche(qmass, abcd(4), 0.0d0, abcd(3), rsep, frot)
-gside = droche(qmass, abcd(4), abcd(2), 0.0d0, rsep, frot)
+gpole = droche(qmass, 0.0d0, 0.0d0, abcd(3), rsep, frot)
+gside = droche(qmass, 0.0d0, abcd(2), 0.0d0, rsep, frot)
 gback = droche(qmass, abcd(4)-abcd(1), 0.0d0, 0.0d0, rsep, frot)
 gfront= droche(qmass, abcd(4)+abcd(1), 0.0d0, 0.0d0, rsep, frot)
 
@@ -902,7 +1101,7 @@ if (verbose >= v_debug) then
   print *,' g_side = ',gside
   print *,' g_back = ',gback
   print *,' g_front= ',gfront
-end if
+endif
 
 xarr = [abcd(4)+abcd(1), abcd(4), abcd(4)-abcd(1)]
 garr = [gfront/gpole-1.0d0,gside/gpole-1.0d0,gback/gpole-1.0d0]
@@ -911,7 +1110,7 @@ if (verbose >= v_debug) then
   print *,'xfit ',xarr
   print *,'gfit ',garr
   print *,'ax = ',ax
-end if
+endif
 gmodel(1:3) = ax
 
 end function gmodel_coeffs
@@ -932,47 +1131,50 @@ double precision function bright(f, g, npar, par, verbose)
 ! par(2) = a
 ! par(3) = b
 ! par(4) = c 
-! par(5) = the
-! par(6) = phi
-! par(7) = rsep
-! int(par(8)) = Limb darkening law
-! par(9)  = Limb darkening coefficient 1
-! par(10) = Limb darkening coefficient 2
-! par(11) = Limb darkening coefficient 3
-! par(12) = Limb darkening coefficient 4
+! par(5) = d 
+! par(6) = inc
+! par(7) = phi
+! par(8) = rsep (separation between centres-of-mass)
 !
-! par(13:15) = Surface gravity model coefficients (if .not.exact_grav)
+! int(par(9)) = ldlaw = code for limb darkening law. 
+! If ldlaw > -1 ...
+! par(10) = Limb darkening coefficient 1
+! par(11) = Limb darkening coefficient 2
+! par(12) = Limb darkening coefficient 3
+! par(13) = Limb darkening coefficient 4
+!
+! par(14:16) = Surface gravity model coefficients (if .not.exact_grav)
 !              OR
-! par(13) = q, mass ratio = mass of companion/mass of star (if exact_grav)
-! par(14) = g0, local surface gravity at the pole (if exact_grav)
-! par(15) = frot, asynchronous rotation factor
+! par(14) = q, mass ratio = mass of companion/mass of star (if exact_grav)
+! par(15) = g0, local surface gravity at the pole (if exact_grav)
+! par(16) = frot, asynchronous rotation factor
 !
-! par(16) = Gravity darkening coefficient (-v in Wood's notation)
-! par(17) = Incident flux from companion
-! par(18) = Heating/reflection model coefficient
-! par(19) = Heating/reflection model exponent
-! par(20) = Heating/reflection linear limb darkening coefficient
-! par(21) = Companion radius/semi-major axis
-! par(22) = lambda = sky-projected angle between orbital and rotation axes.
-! par(23) = vscale = rotation speed at (x,y,z)=(0,b,0) in km/s
-! par(24) = kboost = Doppler boosting factor
-! par(25) = rvflag /= 0 to calculate flux-weighted radial velocity.
-! par(26) = exact_grav /= 0 to use exact calculation of local surface gravity
-! par(27-30) = Coordinate transformation 
+! par(17) = Gravity darkening coefficient (-v in Wood's notation)
+! par(18) = Incident flux from companion
+! par(19) = Heating/reflection model coefficient
+! par(20) = Heating/reflection model exponent
+! par(21) = Heating/reflection linear limb darkening coefficient
+! par(22) = Companion radius/semi-major axis
+! par(23) = lambda = sky-projected angle between orbital and rotation axes.
+! par(24) = vscale = rotation speed at (x,y,z)=(0,b,0) in km/s
+! par(25) = kboost = Doppler boosting factor
+! par(26) = rvflag /= 0 to calculate flux-weighted radial velocity.
+! par(27) = { 0 to flag .not.exact_grav
+!           { potential at sub-stellar point for exact_grav case
+! par(28-31) = Coordinate transformation 
+! par(32)    = 0 to ignore coordinate transformation
+! par(33:npar) = specific intensity values from mu=0 to mu=1 (if ldlaw < -1).
 !
-! Coordinates
-!  Position on the ellipse is "s" positive along the major axis of the star, 
-! "t" parallel to orbital angular momentum vector.
-! Position of companion is determined by sign of rsep, i.e., if rsep is
-! positive then the companion is on the side of the star towards positive s.
-!
-! Position (s,t) on ellipse given by the following transformation of the
-! input coordinates  (if npar >= 30)
-! s = par(27) + par(29)*f + par(30)*g
-! t = par(28) - par(30)*f + par(29)*g
+! Position (s,t) in projected star coordinates s given by the following
+! transformation of the  input coordinates  (if npar >= 31)
+!   s = par(28) + par(30)*f + par(31)*g
+ !  t = par(29) - par(31)*f + par(30)*g
+! otherwise
+!   s = f
+!   t = g
 ! 
 ! **  If you use this transformation, remember to multiply the result of your
-! integration by the Jacobian = par(29)**2 + par(30)**2
+! integration by the Jacobian = par(30)**2 + par(31)**2
 !
 !
 implicit none
@@ -980,15 +1182,19 @@ integer, intent(in)  :: npar, verbose
 double precision, intent(in) :: f,g,par(npar)
 
 ! Local variables
-double precision :: I_0, F_0,s,t,a,b,c,the,phi,rsep,ldc(4)
+double precision :: I_0, F_0,s,t,a,b,c,d,inc,phi,rsep,ldc(4)
 double precision :: gmodel(3),vgrav,H_0,H_1,uheat,rcomp, q, g0, frot
-double precision :: aw,bw,cw,dw
-double precision, save :: costhe,sinthe,sinsqthe,sinsqphi,cossqthe,cossqphi
-double precision, save :: sin2the,sin2phi,sinphi,cosphi
-double precision, save :: phi_save = not_set_dble, the_save = not_set_dble
-double precision :: w,x,y,z,s2,t2,st,a2,b2,c2,mu,lambda,rv,kboost
-double precision :: ldfac,gfac,heat,radpt,r,rsq,cosgam,dcosgam, vscale
+double precision :: ap,bp,cp,dp
+double precision, save :: cosi,sini,sinsqi,sinsqphi,cossqi,cossqphi
+double precision, save :: sin2i,sin2phi,sinphi,cosphi
+double precision, save :: phi_save = not_set_dble, inc_save = not_set_dble
+double precision :: p,x,xp,y,yp,z,zp,s2,t2,st,a2,b2,c2,mu,lambda,rv,kboost
+double precision :: ldfac,gfac,heat,radpt,r,rsq,cosgam,dcosgam
+double precision :: omsini, omcosi, sinl, cosl, pot
 double precision, parameter :: c_kms = iau_c*1.0d-3 ! Speed of light in km/s
+integer,parameter :: nrpar=7
+double precision :: rpar(nrpar)
+double precision, parameter :: rtol=1.0d-4,mutol=1.0d-12
 integer :: ldlaw
 logical :: rvflag,exact_grav
 
@@ -996,71 +1202,73 @@ I_0   = par(1)
 a     = par(2) 
 b     = par(3) 
 c     = par(4) 
-the   = par(5) 
-phi   = par(6) 
-rsep  = par(7) 
-ldlaw = int(par(8))
-ldc(1:4)    = par(9:12) 
-vgrav  = par(16)
-F_0    = par(17) 
-H_0    = par(18)
-H_1    = par(19)
-uheat  = par(20)
-rcomp  = par(21)
-lambda = par(22)
-vscale = par(23)
-kboost = par(24)
-rvflag = par(25).ne.0.0d0
-exact_grav = par(26).ne.0.0d0
+d     = par(5) 
+inc   = par(6) 
+phi   = par(7) 
+rsep  = par(8) 
+ldlaw = int(par(9))
+ldc(1:4)    = par(10:13) 
+pot = par(27)
+exact_grav = pot.ne.0.0d0
 if (exact_grav) then
-  q  = par(13) 
-  g0 = par(14) 
-  frot = par(15) 
+  q  = par(14) 
+  g0 = par(15) 
+  frot = par(16) 
 else
-  gmodel(1:3) = par(13:15)
-end if
-if (npar >= 30) then
-  s = par(27) + par(29)*f + par(30)*g
-  t = par(28) - par(30)*f + par(29)*g
+  gmodel(1:3) = par(14:16)
+  g0 = not_set_dble
+endif
+vgrav  = par(17)
+F_0    = par(18) 
+H_0    = par(19)
+H_1    = par(20)
+uheat  = par(21)
+rcomp  = par(22)
+lambda = par(23)
+omsini = par(24)/b
+kboost = par(25)
+rvflag = par(26).ne.0.0d0
+if (par(32) /= 0) then
+  s = par(28) + par(30)*f + par(31)*g
+  t = par(29) - par(31)*f + par(30)*g
 else
   s = f
   t = g
-end if
+endif
 
-if ((vscale/=0.0d0).and.(lambda/=0.0d0).and.(abs(a-c)>epsilon(0.0d0))) then
+if ((omsini/=0.0d0).and.(lambda/=0.0d0).and.(abs(a-c)>epsilon(0.0d0))) then
   if(verbose >= v_error) then
     print *,'Error in bright - lambda /= 0 for non-spherical star.'
-    print *,lambda,vscale
-    print *,a,b,c,the,phi,s,t
-  end if
+    print *,lambda,omsini
+    print *,a,b,c,inc,phi,s,t
+  endif
   bright = bad_dble
   return
-end if
+endif
 
 ! Dark star, no heating ...
 heat = F_0*H_0
 if ((I_0 == 0.0d0).and.(heat == 0.0d0)) then
   bright = 0
   return
-end if
+endif
 
-if (the /= the_save) then
-  the_save = the
-  costhe = cos(the)
-  sinthe = sin(the)
-  cossqthe = costhe**2
-  sinsqthe = 1.d0-cossqthe
-  sin2the = 2.d0*costhe*sinthe
-end if
+if (inc /= inc_save) then
+  inc_save = inc
+  cosi = cos(inc)
+  sini = sin(inc)
+  cossqi = cosi**2
+  sinsqi = sini**2
+  sin2i = 2.d0*cosi*sini
+endif
 if (phi /= phi_save) then
   phi_save = phi
   cosphi = cos(phi)
   sinphi = sin(phi)
   cossqphi = cosphi**2
-  sinsqphi = 1.d0-cossqphi
+  sinsqphi = sinphi**2
   sin2phi = 2.d0*cosphi*sinphi
-end if
-
+endif
 
 a2 = a**2
 b2 = b**2
@@ -1069,112 +1277,145 @@ s2 = s**2
 t2 = t**2
 st = s*t
 
-aw = sinsqthe*sinsqphi/a2 + sinsqthe*cossqphi/b2 + cossqthe/c2
-bw = (sinthe*sin2phi*s - sin2the*sinsqphi*t)/a2   &
-   - (sinthe*sin2phi*s + sin2the*cossqphi*t)/b2   &
-   + sin2the*t/c2
-cw = (cossqphi*s2 + cossqthe*sinsqphi*t2 - costhe*sin2phi*st)/a2 &
-   + (sinsqphi*s2 + cossqthe*cossqphi*t2 + costhe*sin2phi*st)/b2 &
-   + sinsqthe*t2/c2 - 1.d0
-dw = bw**2 - 4.d0*aw*cw
-if (dw < 0.d0) then
-  ! The is usually a round-off error problem. If the radial distance of the
-  ! point from the origin is within the minimum/maximum possible values then
-  ! assume this is the case.
-  if ((hypot(s,t) > c).and.(hypot(s,t) < a)) then
-    bright = 0.0d0
-    if(verbose >= v_debug) then
-      print *,'bright: No such point on surface.'
-      print *,hypot(s,t)-a,hypot(s,t)-b,hypot(s,t)-c
-      print *,a,b,c,the,phi,dw
-      print *,s,t,npar
-      print *,f,g,par(27:30)
-      print *,aw,bw,cw
-    end if
-    return
-  else
-    if(verbose > v_silent) then
-      print *,'bright: No such point on surface.'
-      print *,hypot(s,t)-a,hypot(s,t)-b,hypot(s,t)-c
-      print *,a,b,c,the,phi,dw
-      print *,s,t,npar
-      print *,f,g,par(27:30)
-      print *,aw,bw,cw
-    end if
-    bright = bad_dble
-    return
-  end if
-end if
+ap = sinsqi*cossqphi/a2 + sinsqi*sinsqphi/b2 + cossqi/c2
+bp = (sini*sin2phi*s - sin2i*cossqphi*t)/a2   &
+   - (sini*sin2phi*s + sin2i*sinsqphi*t)/b2   &
+   + sin2i*t/c2
+cp = (sinsqphi*s2 + cossqi*cossqphi*t2 - cosi*sin2phi*st)/a2 &
+   + (cossqphi*s2 + cossqi*sinsqphi*t2 + cosi*sin2phi*st)/b2 &
+   + sinsqi*t2/c2 - 1.d0
+dp = bp**2 - 4.d0*ap*cp
 
-w = (-bw + sqrt(dw))/(2.d0*aw)
-x = cosphi*s - costhe*sinphi*t + sinthe*sinphi*w
-y = sinphi*s + costhe*cosphi*t - sinthe*cosphi*w
-z =            sinthe*t        + costhe*w
+if (dp < 0.d0) then
+  ! The is usually a round-off error problem. 
+  ! Assume this is the case and do the calculation for the point on the limb
+  if(verbose >= v_debug) then
+    print *,'bright: No such point on surface.'
+    print *,hypot(s,t)-a,hypot(s,t)-b,hypot(s,t)-c
+    print *,a,b,c,inc,phi,dp
+    print *,s,t,npar
+    print *,f,g,par(28:31)
+    print *,ap,bp,cp
+  endif
+  dp = 0
+endif
+
+p = (-bp + sqrt(dp))/(2.d0*ap)
+
+! Coordinates of input point (s,t) in (x', y', z') coordinates centred on
+! geometric centre of ellipsoid
+xp = sinphi*s - cosi*cosphi*t + sini*cosphi*p
+yp = cosphi*s + cosi*sinphi*t - sini*sinphi*p 
+zp =            sini*t        + cosi*p        
+
+! Coordinates of input point (s,t) in (x,y,z) coordinates centred on
+! centre-of-mass of the star
+x = xp+d
+y = yp
+z = zp
 
 ! cosine of the angle between the line of sight and the surface normal to a
 ! triaxial ellipsoid.
-mu = (x*sinphi*sinthe/a2 -y*sinthe*cosphi/b2 + z*costhe/c2) &
-   /       sqrt(x*x/a2/a2 + y*y/b2/b2 + z*z/c2/c2)   
-if ((mu <= 0.d0).or.(mu >= 1.d0)) then 
-  print *,'Error in bright - mu out of range'
-  print *,x,y,z,a2,b2,c2,sinthe,cosphi,mu
-  bright = bad_dble
-  return
-end if
+mu = (xp*cosphi*sini/a2 -yp*sinphi*sini/b2 + zp*cosi/c2) &
+   /       sqrt((xp/a2)**2 + (yp/b2)**2 +(zp/c2)**2)   
+
+if(verbose >= v_debug) then
+  print *,'bright: mu = ',mu
+endif
+
+if (abs(mu) < mutol) then
+    mu = mutol
+else if (abs(mu-1.d0) < mutol) then
+    mu = 1-mutol
+else if ((mu < 0.d0).or.(mu > 1.d0)) then
+    print *,'Error in bright - mu out of range'
+    print *,a,b,c,inc,phi,dp
+    print *,s,t,npar
+    print *,xp,y,z,a2,b2,c2
+    print *,sini,cosi,sinphi,cosphi,mu
+    bright = bad_dble
+    return
+endif
 
 ! Limb darkening
-ldfac = limbdark(mu, ldlaw, ldc)
+if (ldlaw < -1) then
+  ldfac = limbdark(mu, ldlaw, par(33:npar))
+else
+  ldfac = limbdark(mu, ldlaw, ldc)
+endif
 if (ldfac == bad_dble) then
   if(verbose >= v_error) then
     print *,'Error in bright - invalid parameters for limbdark.'
     print *,ldlaw,ldc
-  end if
+    print *,par(33),par(npar)
+  endif
   bright = bad_dble
   return
-end if
+endif
 
-if (exact_grav) then
-  gfac = (droche(q, x, y, z, abs(rsep), frot)/g0)**vgrav
+if (vgrav /= 0.0d0) then
+  if (exact_grav) then
+    rpar(1) = pot
+    rpar(2) = x
+    rpar(3) = y
+    rpar(4) = z
+    rpar(5) = rsep
+    rpar(6) = q
+    rpar(7) = frot
+    r = zbrent(func_r,0.9d0,1.1d0,rtol,nrpar,rpar,verbose-1)
+    gfac = (droche(q, r*x, r*y, r*z, rsep, frot)/g0)**vgrav
+  else
+    gfac =  (poly(gmodel(1:3),x)*(1.0d0-z**2/c2) + 1.0d0)**vgrav
+  endif
 else
-  gfac =  (poly(gmodel(1:3),x)*(1.0d0-z**2/c2) + 1.0d0)**vgrav
-end if
+  gfac = 1
+endif
 
 ! Heating/reflection
 if (heat /=  0.d0) then
-  radpt   = sqrt(x**2 + y**2 + z**2)
-  rsq =  radpt**2 - 2.d0*x*rsep + rsep**2
+  radpt   = sqrt(xp**2 + y**2 + z**2)
+  rsq =  radpt**2 - 2.d0*xp*(rsep-d) + (rsep-d)**2
   r =  sqrt(rsq)
-  cosgam = (x*rsep/radpt - radpt)/r
+  cosgam = (xp*(rsep-d)/radpt - radpt)/r
   dcosgam = (radpt-rcomp)/r
   if (cosgam < -dcosgam) then
     heat = 0
   else if (cosgam > dcosgam) then
-    heat = heat*(1.0/rsq)**H_1*(1.d0-uheat*(1.d0-mu)) 
+    heat = heat*(1.0d0/rsq)**H_1*(1.d0-uheat*(1.d0-mu)) 
   else 
     heat = heat*(0.5d0*(cosgam+dcosgam)/dcosgam/rsq)**H_1*(1.d0-uheat*(1.d0-mu))
-  end if
-end if
+  endif
+endif
 
 bright = I_0*ldfac*gfac+heat
-if (vscale /= 0.0d0) then
+if (omsini /= 0.0d0) then
   if (lambda == 0.0d0) then
-    rv = vscale*s/b
+    rv = omsini*s
   else
-    rv = vscale*(cos(lambda)*s - sin(lambda)*t)/b
-  end if
+    cosl = cos(lambda)
+    sinl = sin(lambda)
+    if (sini /= 0.0d0) then
+      omcosi = omsini/sini*cosi
+    else
+      omcosi = bad_dble
+    endif
+    rv = omsini*(cosl*s - sinl*t) 
+  endif
 
   if (rvflag) then
     bright = rv*bright*(1.0d0 - kboost*rv/c_kms)
   else
     bright = bright*(1.0d0 - kboost*rv/c_kms)
-  end if
+  endif
 else
   rv = 0
-end if
+endif
 
 if(verbose >= v_debug) then
-  print *,'bright,s,t,rv,ldfac,gfac,heat= ',bright ,s,t,rv,ldfac,gfac,heat
-end if
+  print '(a,11G13.4)','bright,s,t,p,xp,y,z,rv,inc,phi,dp = ',bright,s,t,p &
+   ,xp,y,z,rv,inc ,dp
+   ! phi,(xp/a)**2+(y/b)**2+(z/c)**2 - 1.0d0
+endif
 
 return
 
@@ -1341,7 +1582,7 @@ if (incl /= halfpi) then
             theta, verbose1)
 else
   theta = theta_0
-end if
+endif
 if (verbose >= v_debug) print *,'t_ecl_to_peri: theta = ',theta
 
 ! (4.10)
@@ -1349,7 +1590,7 @@ if (theta == pi) then
   ee = pi
 else
  ee = 2.0d0 * atan(sqrt((1.0d0-ecc)/(1.0d0+ecc)) * tan(theta/2.0d0))
-end if
+endif
 eta = ee - ecc*sin(ee)
 delta_t = eta*p_sid/twopi
 t_ecl_to_peri = t_ecl  - delta_t
@@ -1370,6 +1611,7 @@ sin2i = par(2)
 omega = par(3)
 ecc   = par(4)
 delta_func = efac*sqrt(1.0d0 - sin2i*sin(theta+omega)**2)/(1.0d0+ecc*cos(theta))
+if (verbose >= v_debug) print *,par,delta_func
 end function delta_func
 
 end module stellar
