@@ -49,15 +49,17 @@ public midangle          ! Bisector of two angles
 public triangle_area     ! Area of a triangle defined by three points
 public arc_area          ! Area of an arc on a unit circle
 public ucircle_line_intersect ! Intersection of a line with unit circle
-public circles_intersect ! Intersection points of two circles
+public circles_intersect  ! Intersection points of two circles on a plane
+public sph_circ_intersect ! Intersection points of two circles on a sphere
 public distance          ! Separation between two points in 2-d Euclidean space
 public angular_distance  ! Great circle distance between points on a sphere.
 public parfit            ! Parabolic fit to three points
 public unitfunc          ! =1
 public verbose_for_calls ! Set verbose value for function calls.
 public acot              ! Inverse cotangent function
+public cross3            ! Cross product of 3-element vectors
 
-! Bit flags for circles_intersect
+! Bit flags for circles_intersect and sph_circ_intersect
 integer, parameter, public :: b_circles_no_overlap         = 0
 integer, parameter, public :: b_circles_1_inside_2         = 1
 integer, parameter, public :: b_circles_2_inside_1         = 2
@@ -108,7 +110,7 @@ if (present(reverse)) then
   reverse_l = reverse
 else
   reverse_l = .false.
-end if
+endif
 
 if (reverse_l) then
   do j=1,size(a)-1
@@ -122,7 +124,7 @@ else
     l = j-1+k
     if (j /= l) call swap(a(l),a(j))
   end do
-end if
+endif
 return
 end subroutine slowsort
 
@@ -141,7 +143,7 @@ subroutine heapsort(xdata, key)
   if (size(key) < ndata) then
    key(1) = bad_int
    return
-  end if
+  endif
 
   do i = 1, ndata 
     key(i) = i 
@@ -163,21 +165,21 @@ subroutine heapsort(xdata, key)
       if(ir == 1) then 
         key(1) = ik 
         return 
-      end if 
-    end if 
+      endif 
+    endif 
     i = l 
     j = l + l 
     do while (j <= ir) 
       if(j < ir) then 
         if(xdata(key(j)) < xdata(key(j+1))) j = j + 1 
-      end if 
+      endif 
       if(x < xdata(key(j))) then 
         key(i) = key(j) 
         i = j 
         j = j + j 
       else 
         j = ir + 1 
-      end if 
+      endif 
     end do
     key(i) = ik 
   end do 
@@ -199,7 +201,7 @@ if (present(reverse)) then
   reverse_l = reverse
 else
   reverse_l = .false.
-end if
+endif
 
 n = size(a)
 
@@ -278,7 +280,7 @@ if (reverse_l) then
       end do
     end select
 
-    end if 
+    endif 
 
     return
     end function poly
@@ -323,9 +325,9 @@ if (fa*fb > 0.0d0) then
     print *, 'zbrent: input values do not bracket root'
     print *, a,b
     print *, fa,fb
-  end if
+  endif
   return
-end if
+endif
 
 fc = fb
 do iter=1, itmax
@@ -334,7 +336,7 @@ do iter=1, itmax
     fc = fa
     d = b-a
     e=d
-  end if
+  endif
   if (abs(fc) < abs(fb)) then
     a = b
     b = c
@@ -342,13 +344,13 @@ do iter=1, itmax
     fa = fb
     fb = fc
     fc = fa
-  end if
+  endif
   tol1 = 2.0d0*eps*abs(b)+0.5d0*tol
   xm = (c-b)/2.0d0
   if (abs(xm) <  tol1 .or. fb == 0.0d0) then
     zbrent = b
     return
-  end if
+  endif
   if (abs(e) > tol1 .and. abs(fa) >  abs(fb)) then
     s = fb/fa
     if (a == c) then
@@ -359,7 +361,7 @@ do iter=1, itmax
       r = fb/fc
       p = s*(2.0d0*xm*q*(q-r)-(b-a)*(r-1.0d0))
       q = (q-1.0d0)*(r-1.0d0)*(s-1.0d0)
-    end if
+    endif
     if (p > 0.0d0) q = - q
     p = abs(p)
     if (2.0d0*p < min(3.0d0*xm*q-abs(tol1*q),abs(e*q))) then
@@ -368,18 +370,18 @@ do iter=1, itmax
     else
       d = xm
       e = d
-    end if
+    endif
   else
     d = xm
     e = d
-  end if
+  endif
   a = b
   fa = fb
   if( abs(d) > tol1) then
     b = b + d
   else
     b = b + sign(tol1, xm)
-  end if
+  endif
   fb = func(b,npar,par,verbose-1)
   if (fb == bad_dble) return
 end do
@@ -387,7 +389,7 @@ if (verbose >= v_error) then
   print *, 'zbrent exceeded maximum iterations.'
   print *, a,b
   print *, fa,fb
-end if
+endif
 return
 end function zbrent
 
@@ -442,9 +444,9 @@ if (verbose >= v_error) then
     print *, 'rtsafe: input values do not bracket root'
     print *, x1, x2
     print *, fl,fh
-  end if
+  endif
   return
-end if
+endif
 
 if (fl == 0.0d0) then
   rtsafe = x1
@@ -458,7 +460,7 @@ else if (fl < 0.0d0) then
 else
   xh = x1
   xl = x2
-end if
+endif
 
 rtsafe = 0.5d0*(x1+x2)
 dxold  =abs(x2-x1)
@@ -480,7 +482,7 @@ do j=1,maxit
     temp=rtsafe
     rtsafe=rtsafe-dx
     if(temp == rtsafe) return
-  end if
+  endif
   if (abs(dx) < tol) return  ! Convergence criterion.
   f= func(rtsafe,npar, par, verbose-1)
   ! Added a test here to return if abs(f) < ftol
@@ -490,30 +492,30 @@ do j=1,maxit
     xl = rtsafe
   else 
     xh=rtsafe
-  end if
+  endif
 end do
 if (verbose >= v_error) then
   print *, 'rtsafe: Failed to converge'
   print *, xl, xh
   print *, x1, x2
   print *, f, df,ftol
-end if
+endif
 return
 end function rtsafe
 
 !-----------------------------------------------------------------------------
 
 double complex function laguerre(a,m,x)
-! Given a degree m and the complex coefficients a(1:m+1) of a the polynomial
+! Given a degree m and the complex coefficients a(1:m+1) of the polynomial
 ! sum(i=1,m+1) a(i) * x**(i-1), and given a complex value x, this routine
 ! improves x by Laguerre's method until it converges, within the achievable
 ! round-off limit, to a root of the given polynomial. 
-!  If the algorithm fails to converge return value is bad_dble
+!  If the algorithm fails to converge return value is (bad_dble, bad_dble)
 ! 
 ! Follows the algorithm described in Numerical Recipes in Fortran 90
 ! (Press et al., http://www.nr.com)
 !
-integer, intent(in)     :: m      ! Degree of the polynomial
+integer, intent(in)        :: m      ! Degree of the polynomial
 double complex, intent(in) :: a(m+1) ! Polynomial coefficients (ascending power)
 double complex, intent(in) :: x      ! Initial/final estimate of root
 
@@ -561,21 +563,22 @@ do iter=1,maxit
       dx = m/gp
     else
       dx = exp(dcmplx(log(1.0d0+abx),dble(iter)))
-    end if
-  end if
+    endif
+  endif
   x1 = laguerre - dx
   if (laguerre == x1) return
   if (mod(iter,mt) /= 0) then
     laguerre = x1
   else
     laguerre = laguerre - dx*frac(iter/mt)
-  end if
+  endif
 end do
 laguerre = dcmplx(bad_dble,bad_dble)
 return
 end function laguerre
 
-!-------------------------------------------------------------------------------
+
+!-----------------------------------------------------------------------------
 
 function affine(x,tr,inverse) result(xt)
 implicit none
@@ -604,20 +607,20 @@ if (present(inverse)) then
   inverse_l = inverse
 else
   inverse_l = .false.
-end if
+endif
 
 if (inverse_l) then
   d = tr(1,1)*tr(2,2) -  tr(1,2)*tr(2,1) 
   if (d == 0.0d0) then
     xt(:) = bad_dble
     return
-  end if
+  endif
   xt(1) = ( tr(2,2)*(x(1)-tr(1,3)) - tr(1,2)*(x(2)-tr(2,3)))/d
   xt(2) = (-tr(2,1)*(x(1)-tr(1,3)) + tr(1,1)*(x(2)-tr(2,3)))/d
 else
   xt(1) = tr(1,3) + tr(1,1)*x(1) + tr(1,2)*x(2)
   xt(2) = tr(2,3) + tr(2,1)*x(1) + tr(2,2)*x(2)
-end if
+endif
 end function affine
 
 !-----------------------------------------------------------------------------
@@ -686,7 +689,7 @@ if (present(reflex)) then
   reflex_l = reflex
 else
   reflex_l = .false.
-end if
+endif
 
 if (reflex_l) then
 
@@ -694,7 +697,7 @@ if (reflex_l) then
     midangle = 0.5d0*(th0+th1)
   else 
     midangle = mod(pi+0.5d0*(th0+th1), twopi)
-  end if
+  endif
 
 else
 
@@ -703,9 +706,9 @@ else
     if (midangle.lt.0.d0) midangle = midangle + twopi
   else 
     midangle = 0.5d0*(th0+th1)
-  end if
+  endif
 
-end if
+endif
 
 end function midangle
                                                                         
@@ -727,7 +730,7 @@ if (present(signed)) then
   signed_l = signed
 else
   signed_l = .false.
-end if
+endif
 
 if (signed_l) then
  triangle_area = 0.5d0*(p_1(1)*(p_2(2)-p_3(2)) &
@@ -735,7 +738,7 @@ if (signed_l) then
 else
  triangle_area = abs(0.5d0*(p_1(1)*(p_2(2)-p_3(2)) &
                - p_2(1)*(p_1(2)-p_3(2)) + p_3(1)*(p_1(2)-p_2(2))))
-end if
+endif
 
 return
 end function triangle_area
@@ -777,7 +780,7 @@ else
   w = -(b + sign(r,b))
   t(1) = 2.0d0*c/w
   t(2) = 0.5d0*w/a
-end if
+endif
 return 
 end function ucircle_line_intersect
                                                                         
@@ -805,7 +808,7 @@ end function arc_area
 !-------------------------------------------------------------------------------
 
 subroutine circles_intersect(circle_1, circle_2, xy, circles_flags)
-! Number and positions of interesections points of two circles.
+! Number and positions of intersections points of two circles on a plane
 implicit none
 double precision, intent(in)  :: circle_1(3) ! (x_c, y_c) and radius of circle 1
 double precision, intent(in)  :: circle_2(3) ! (x_c, y_c) and radius of circle 2
@@ -829,32 +832,36 @@ r_1 = circle_1(3)
 r_2 = circle_2(3)
 circles_flags = 0 
 if ((r_1 <= 0.0d0).or.(r_2 <= 0.0d0)) then
+  xy(:,:) = not_set_dble
   circles_flags = ibset(circles_flags, b_circles_invalid)
   return
-end if
+endif
 
 dx = circle_2(1) - circle_1(1)
 dy = circle_2(2) - circle_1(2)
 d = hypot(dx,dy)
 
 if ((d == 0.0d0).and.(r_1 == r_2)) then
+  xy(:,:) = not_set_dble
   circles_flags = ibset(circles_flags, b_circles_identical)
   return
-end if
+endif
 
 if (d > (r_1 + r_2)) then
+  xy(:,:) = not_set_dble
   circles_flags = ibset(circles_flags, b_circles_no_overlap)
   return
-end if
+endif
 
 if (d < abs(r_2 - r_1)) then
+  xy(:,:) = not_set_dble
   if (r_1 < r_2) then
     circles_flags = ibset(circles_flags, b_circles_1_inside_2)
   else
     circles_flags = ibset(circles_flags, b_circles_2_inside_1)
   endif
   return
-end if
+endif
 
 a = (r_1**2 - r_2**2 + d**2) / (2.0d0 * d)
 x2 = circle_1(1) + dx * a/d
@@ -866,7 +873,7 @@ if (d == (r_1 + r_2)) then
   circles_flags = ibset(circles_flags, b_circles_tangent)
   circles_flags = ibset(circles_flags, b_circles_no_overlap)
   return
-end if
+endif
 
 if (d == abs(r_1 - r_2)) then
   xy(1:2,1) = (/ x2, y2 /)
@@ -878,7 +885,7 @@ if (d == abs(r_1 - r_2)) then
     circles_flags = ibset(circles_flags, b_circles_2_inside_1)
   endif
   return
-end if
+endif
 
 h = sqrt(r_1**2 - a**2)
 rx = -dy * h/d
@@ -891,6 +898,102 @@ return
 
 end subroutine circles_intersect
                                                          
+                                                                        
+!-------------------------------------------------------------------------------
+
+subroutine sph_circ_intersect(circle_1, circle_2, xpts, circles_flags)
+! Number and positions of intersections points of two circles on a sphere
+! See https://gis.stackexchange.com/questions/48937/calculating-intersection-of-two-circles
+! All input/output angles are in radians, including the radii of the circles.
+implicit none
+double precision, intent(in)  :: circle_1(3) ! (lon, lat) and radius of circle 1
+double precision, intent(in)  :: circle_2(3) ! (lon, lat) and radius of circle 2
+double precision, intent(out) :: xpts(2,2)   ! (lon,lat) for intersections 
+integer, intent(out)  :: circles_flags ! Bit mask for flags
+! circles_flags is a bit mask with the following flags
+! b_circles_no_overlap  (but may have a tangent point)
+! b_circles_1_inside_2  (and may have a tangent point)
+! b_circles_2_inside_1  (and may have a tangent point)
+! b_circles_tangent     (there is a tangent point, xy(1:2,1) = xy(1:2,2)
+! b_circles_intersect   (no tangent points, two distinct intersection points)
+! b_circles_identical   (xy not defined in this case)
+! b_circles_invalid     (parameters invalid for either circle)
+!
+! To test these bit flags, use btest(circles_flags, <bit flag name>)
+
+! Local variables
+double precision :: x_1(3), x_2(3), n(3), x_0(3)
+double precision :: r_1, r_2, q, d, a, b, t
+
+circles_flags = 0 
+r_1 = circle_1(3)
+r_2 = circle_2(3)
+
+if ( (r_1 <= 0.0d0).or.(r_1 >= halfpi).or. &
+     (r_2 <= 0.0d0).or.(r_2 >= halfpi) ) then
+  xpts(:,:) = not_set_dble
+  circles_flags = ibset(circles_flags, b_circles_invalid)
+  return
+endif
+
+d = angular_distance(circle_1(1),circle_1(2),circle_2(1),circle_2(2))
+
+if ((d <  epsilon(0.0d0)).and.(abs(r_1 - r_2)< epsilon(0.0d0))) then
+    xpts(:,:) = not_set_dble
+    circles_flags = ibset(circles_flags, b_circles_identical)
+    return
+endif
+
+if (d > (r_1+r_2)) then
+  xpts(:,:) = not_set_dble
+  circles_flags = ibset(circles_flags, b_circles_no_overlap)
+  return
+endif
+
+if (d < abs(r_2 - r_1)) then
+  xpts(:,:) = not_set_dble
+  if (r_1 < r_2) then
+    circles_flags = ibset(circles_flags, b_circles_1_inside_2)
+  else
+    circles_flags = ibset(circles_flags, b_circles_2_inside_1)
+  endif
+  return
+endif
+
+
+x_1 = (/cos(circle_1(1))*cos(circle_1(2)), &
+        sin(circle_1(1))*cos(circle_1(2)), &
+        sin(circle_1(2)) /)
+x_2 = (/cos(circle_2(1))*cos(circle_2(2)), &
+        sin(circle_2(1))*cos(circle_2(2)), &
+        sin(circle_2(2)) /)
+
+q = dot_product(x_1, x_2)
+a = (cos(r_1) - cos(r_2)*q) / (1.0d0 - q**2)
+b = (cos(r_2) - cos(r_1)*q) / (1.0d0 - q**2)
+
+n = cross3(x_1, x_2)
+
+x_0 = a*x_1 + b*x_2
+
+t = sqrt((1.0d0 - min(1.0d0,dot_product(x_0,x_0)))/dot_product(n,n))
+
+if (t == 0.0d0) then
+    circles_flags = ibset(circles_flags, b_circles_tangent)
+endif
+
+x_1 = x_0 + t*n
+x_2 = x_0 - t*n
+
+xpts(1,1) =  atan2(x_1(2), x_1(1))
+xpts(2,1) =  atan2(x_1(3), hypot(x_1(1), x_1(2))) 
+xpts(1,2) =  atan2(x_2(2), x_2(1))
+xpts(2,2) =  atan2(x_2(3), hypot(x_2(1), x_2(2))) 
+      
+return
+
+end subroutine sph_circ_intersect
+                                                         
 !-------------------------------------------------------------------------------
 
 double precision function distance(xy_1, xy_2) 
@@ -902,11 +1005,18 @@ end function distance
 
 !-------------------------------------------------------------------------------
 
-double precision function angular_distance(lat_1, lon_1, lat_2, lon_2)
+double precision function angular_distance(lon_1, lat_1, lon_2, lat_2)
+! Great circle distance in radians between points (lon1, lat1) and 
+! (lon2, lat2), also in radians. 
+! See https://www.movable-type.co.uk/scripts/gis-faq-5.1.html
 implicit none
-double precision, intent(in)  :: lat_1, lon_1, lat_2, lon_2
-angular_distance = acos(cos(lat_1)*cos(lat_2)*cos(lon_1-lon_2) + &
-                        sin(lat_1)*sin(lat_2))
+double precision, intent(in)  :: lon_1, lat_1, lon_2, lat_2
+
+double precision :: dlon, dlat, a
+dlon = lon_2-lon_1
+dlat = lat_2-lat_1
+a = sin(0.5d0*dlat)**2 + cos(lat_1)*cos(lat_2)*sin(0.5d0*dlon)**2
+angular_distance = 2.0d0 * asin(min(1.0d0,sqrt(a)))
 return
 end function angular_distance
 
@@ -930,7 +1040,7 @@ d =  dv1*(dx(1)-dx(2)) - dv2*(dx(1)-dx(3))
 if (abs(d) < epsilon(0.0d0)) then
   a(1:3) = bad_dble
   return
-end if
+endif
 a(1) = ( dv1*(dx(1)*dy(2)-dx(2)*dy(1)) -  &
          dv2*(dx(1)*dy(3)-dx(3)*dy(1)) )/d
 
@@ -940,7 +1050,7 @@ d =  dv1*(dx(2)-dx(1)) - dv2*(dx(3)-dx(1))
 if (abs(d) < epsilon(0.0d0)) then
   a(1:3) = bad_dble
   return
-end if
+endif
 a(2) = ( dv1*(dy(2)-dy(1)) - dv2*(dy(3)-dy(1)) )/d
 
 dv1 = dx(3) - dx(1)
@@ -949,7 +1059,7 @@ d =  dv1*(dx(2)*dx(2)-dx(1)*dx(1)) - dv2*(dx(3)*dx(3)-dx(1)*dx(1))
 if (abs(d) < epsilon(0.0d0)) then
   a(1:3) = bad_dble
   return
-end if
+endif
 a(3) = ( dv1*(dy(2)-dy(1)) - dv2*(dy(3)-dy(1)) )/d
       
 return
@@ -975,7 +1085,7 @@ if (verbose >= v_debug) then
   verbose_for_calls = verbose -1
 else
   verbose_for_calls = verbose
-end if
+endif
 end function verbose_for_calls
 
 !----------------------------------------------------------------------
@@ -1015,9 +1125,9 @@ case(5)
   px = x*(x*(x*(x*p(5)+p(4))+p(3))+p(2))+p(1)
   dpdx = x*(x*(x*p(5)*4.d0+p(4)*3.d0)+p(3)*2.d0)+p(2)
 case  default
-  px = p(np)
-  dpdx = 0.d0
-  do i=np-1,1,-1
+  px = p(np)*x + p(np-1)
+  dpdx = p(np)
+  do i=np-2,1,-1
     dpdx = dpdx*x*px
     px = px*x + p(i)
   end do
@@ -1065,7 +1175,8 @@ interface
 end interface
 
 verbose1 = verbose_for_calls(verbose)
-
+brent = not_set_dble
+d = not_set_dble
 a = min(ax, cx)
 b = max(ax, cx)
 v = bx
@@ -1083,7 +1194,7 @@ do iter = 1, itmax
     xmin = x
     brent = fx
     return
-  end if
+  endif
   if(abs(e) > tol1) then
     r = (x-w)*(fx-fv)
     q = (x-v)*(fx-fw)
@@ -1101,11 +1212,11 @@ do iter = 1, itmax
        d = p/q
        u=x+d
        if ( u-a < tol2 .or. b-u < tol2) d = sign(tol1, xm-x)
-     end if
+     endif
    else
      e = merge(a-x, b-x, x >= xm)
      d = cgold*e
-   end if
+   endif
    u = merge(x+d, x+sign(tol1,d), abs(d) >= tol1)
    fu = f(u,npar,par,verbose1)
    if (fu <= fx) then
@@ -1113,7 +1224,7 @@ do iter = 1, itmax
        a = x
      else
        b = x
-     end if
+     endif
      call shft(v,w,x,u)
      call shft(fv,fw,fx,fu)
    else
@@ -1121,7 +1232,7 @@ do iter = 1, itmax
        a = u
      else
        b = u
-     end if
+     endif
      if (fu <= fw .or. w == x) then
        v=w
        fv=fw
@@ -1130,10 +1241,19 @@ do iter = 1, itmax
      else if (fu <= fv .or. v == x .or. v == w) then
        v=u
        fv=fu
-     end if
-   end if
+     endif
+   endif
  end do
- if (verbose >= v_warn) print *,'brent: failed to converge'
+ if (verbose >= v_warn) then
+   print *,'brent: failed to converge'
+   print *, v,w,x,u
+   print *, fv,fw,fx,fu
+   print *, abs(x-xm), tol1, tol2
+ endif
+ xmin = x
+ brent = fx
+ return
+
  contains
 
  subroutine shft(a,b,c,d)
@@ -1147,16 +1267,30 @@ do iter = 1, itmax
  end function brent
 
 !----------------------------------------------------------------------
+
 double precision function acot(x)
 implicit none
-double precision :: x  ! Input value
+double precision, intent(in) :: x  ! Input value
 
 if (x < 0.d0 ) then
   acot = -halfpi - atan(x)
 else
   acot =  halfpi - atan(x)
-end if 
+endif 
 end function acot
 
+!----------------------------------------------------------------------
+
+function cross3(a, b) result(x)
+implicit none
+double precision, intent(in) :: a(3), b(3)
+double precision :: x(3)
+
+x = (/ a(2) * b(3) - a(3) * b(2), &
+       a(3) * b(1) - a(1) * b(3), &
+       a(1) * b(2) - a(2) * b(1) /)
+
+end function cross3
+
 !-----------------------------------------------------------------------------
 end module utils
