@@ -30,20 +30,21 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
        didt = None,
        domdt = None,
        rotfac_1 = 1, rotfac_2 = 1,
+       hf_1 = 1.5, hf_2 = 1.5,
        bfac_1 = None, bfac_2 = None,
        heat_1 = None, heat_2 = None, 
        lambda_1 = None, lambda_2 = None,
        vsini_1 = None, vsini_2 = None,
        t_exp=None, n_int=None, 
-       grid_1='sparse', grid_2='sparse',
+       grid_1='default', grid_2='default',
        ld_1=None, ld_2=None,
        shape_1='sphere', shape_2='sphere',
-       spots_1=None, spots_2=None):
+       spots_1=None, spots_2=None,
+       exact_grav=False, verbose=1):
   """
-  Calculate the light curve of a binary star
+  Calculate the fluxed emitted from each star in an eclipsing binary star
 
-  This function calculates the light curve of a binary star using the ellc
-  binary star model [1].
+  This function calculates the fluxes using the ellc binary star model [1].
 
   Parameters
   ----------
@@ -134,6 +135,16 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
       Asynchronous rotation factor for star 2, F_2
       Default is 1.
 
+  hf_1 : float, optional
+      h_f, fluid second Love number for radial displacement, for star 1.
+      Only used if shape_1 = 'love'
+      Default is 1.5.
+
+  hf_2 : float, optional
+      h_f, fluid second Love number for radial displacement, for star 2.
+      Only used if shape_2 = 'love'
+      Default is 1.5.
+
   bfac_1 : {None, float}, optional
       Doppler boosting factor, star 1
       N.B. Doppler boosting is not calculated is parameter a is None
@@ -145,15 +156,15 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
       Default is None.
       
   heat_1 : {None, scalar, 3-tuple of floats}, optional
-      If scalar, coefficient of simplified reflection model.
-      If 3-tuple, parameters of heating+reflection model for star 1, [H_0, H_1, u_H]
+      If scalar, coefficient of simplified reflection model - see Notes.
+      If 3-tuple, parameters of heating+reflection model [H_0, H_1, u_H]
       H_0 is the coefficient, H_1 is the exponent and u_H is the linear
       limb-darkening coefficient.
       Default is None.
   
   heat_2 : {None, scalar, 3-tuple of floats}, optional
-      If scalar, coefficient of simplified reflection model.
-      If 3-tuple, parameters of heating+reflection model for star 1, [H_0, H_1, u_H]
+      If scalar, coefficient of simplified reflection model - see Notes.
+      If 3-tuple, parameters of heating+reflection model [H_0, H_1, u_H]
       H_0 is the coefficient, H_1 is the exponent and u_H is the linear
       limb-darkening coefficient.
       Default is None.
@@ -171,12 +182,12 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
 
   vsini_1  : {None, float}, optional
       V_rot.sini for calculation of R-M effect for star 1 [km/s]
-      See notes below.
+      Not used for this routine, but kept for consistency with lc.py and rv.py
       Default is None.
 
   vsini_2  : {None, float}, optional
       V_rot.sini for calculation of R-M effect for star 2 [km/s]
-      See notes below.
+      Not used for this routine, but kept for consistency with lc.py and rv.py
       Default is None.
 
   t_exp : {None, float, array_like}, optional
@@ -195,25 +206,36 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
 
   grid_1 : {"very_sparse", "sparse", "default", "fine", "very_fine"}, optional
       Grid size used to calculate the flux from star 1.
-      Default is "sparse"
+      Default is "default"
 
   grid_2 : {"very_sparse", "sparse", "default", "fine", "very_fine"}, optional
       Grid size used to calculate the flux from star 2.
-      Default is "sparse"
+      Default is "default"
 
-  ld_1 : {None, "lin", "quad", "sing", "claret", "log", "sqrt", "exp"} 
+  ld_1 : {None, "lin", "quad", "sing", "claret", "log", "sqrt", "exp",
+          "power-2", "mugrid"} 
    Limb darkening law for star 1
+   The "power-2" limb darkening law is taken from  Morello et al., 
+   2017AJ....154..111M:
+    I_lambda(mu)/I_lambda(1) = 1 - ldc_1[0] * (1 - mu**ldc_1[1])
+   For the option "mugrid", ldc_1 must be an array of specific intensity
+   values as a function of mu=cos(theta), where theta is the angle between the
+   line of sight and the normal to a given point of the stellar surface. The
+   value of mu corresponding to element i of ldc_1 is i/(len(ldc_1)-1),  where
+   i=0,1, ... len(ldc_1)-1.  
    Default is None
 
-  ld_2 : {None, "lin", "quad", "sing", "claret", "log", "sqrt", "exp"} 
-   Limb darkening law for star 2
+  ld_2 : {None, "lin", "quad", "sing", "claret", "log", "sqrt", "exp",
+          "power-2", "mugrid"} 
+   Limb darkening law for star 2.
+   See ld_1 for description of the power-2 and mugrid options.
    Default is None
 
-  shape_1 : {"roche", "sphere", "poly1p5", "poly3p0"}
+  shape_1 : {"roche_v", "roche", "sphere", "poly1p5", "poly3p0", "love"}
       Model used to calculate the shape of star 1 - see Notes. 
       Default is "sphere".
 
-  shape_2 : {"roche", "sphere", "poly1p5", "poly3p0"}
+  shape_2 : {"roche_v", "roche", "sphere", "poly1p5", "poly3p0", "love"}
       Model used to calculate the shape of star 2 - see Notes. 
       Default is "sphere".
 
@@ -227,6 +249,11 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
    are latitude, longitude, size and brightness factor. All three angles are
    in degrees.
 
+  exact_grav : {True|False}
+    Use point-by-point calculation of local surface gravity for calculation of
+    gravity darkening is True, otherwise use (much faster) approximation based
+    on functional form fit to local gravity at 4 points on the star.
+
   Returns
   -------
   flux1, flux2, : ndarray, ndarray
@@ -264,6 +291,16 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
    assumed to be independent. The tidal distortion for polytropes is from
    Chandrasekhar [4] and the rotational distortion is calculated by
    interpolation in Table 1 of James [5]. 
+
+    In eccentric orbits, the volume of the star is assumed to be constant. In
+   general, the volume is calculated from the volume of the approximating
+   ellipsoid. In the case of synchronous rotation, the volume of the star can
+   be calculated using equation (2.18) from Kopal "Dynamics of Close Binary
+   Systems" (Springer, 1978) by selecting the star shape model "roche_v".
+    
+    The simplified reflection model is approximately equivalent to Lambert
+   law scattering with the coefficients heat_1 and heat_2  being equal to
+   A_g/2, where A_g is the geometric albedo.
     
 
   Example
@@ -273,7 +310,7 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
   >>> import matplotlib.pyplot as plt
   >>> t = np.arange(-0.25,0.75, 0.001)
   >>> spots_1 = [[30,180],[45,-45],[25,35],[0.2,0.8]]
-  >>> flux1,flux2 = ellc.fluxes(t,radius_1=0.1,radius_2=0.05,sbratio=0.2, \
+  >>> flux1,flux2 = ellc.fluxes(t,radius_1=0.1,radius_2=0.05,sbratio=0.2,
   ...   incl=89.95,q=0.5,ld_1='quad',ldc_1=[0.65,0.2],ld_2='lin',ldc_2=0.45,
   ...   shape_1='poly3p0',shape_2='poly1p5',spots_1=spots_1)
   >>> plt.plot(t,flux1)
@@ -284,11 +321,12 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
   References
   ----------
   .. [1] Maxted, P.F.L. 2016. A fast, flexible light curve model for detached
-      eclipsing binary stars and transiting exoplanets. A&A VOLUME, ARTICLE.
+      eclipsing binary stars and transiting exoplanets. A&A 591, A111, 2016.
   .. [2] Eker, 1994, ApJ, 420, 373.
   .. [3] Wilson, 1979, ApJ, 234, 1054.
   .. [4] Chandrasekhar, 1933, MNRAS, 93, 449.
   .. [5] James, 1964, ApJ, 140, 552.
+  .. [6] Correia, 2014, A&A 570, L5.
 
   """
 
@@ -315,9 +353,11 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
     "quad"   :  2,
     "sing"   :  3,
     "claret" :  4,
-    "log"    : -1,
-    "sqrt"   : -2,
-    "exp"    : -3
+    "log"    :  5,
+    "sqrt"   :  6,
+    "exp"    :  7,
+    "power-2":  8,
+    "mugrid" : -1
   }
   if ld_1 is None:
     ldstr_1 = 'none'
@@ -332,15 +372,18 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
   l1 = ldstr_to_ldcode.get(ldstr_1,None)
   if l1 is None:
     raise Exception("Invalid limb darkening law name")
+
   l2 = ldstr_to_ldcode.get(ldstr_2,None)
   if l2 is None:
     raise Exception("Invalid limb darkening law name")
 
   shapename_to_shapecode = {
+    "roche_v" : -2,
     "roche"   : -1,
     "sphere"  :  0,
     "poly1p5" :  1,
     "poly3p0" :  2,
+    "love"    :  3
   }
   s1 = shapename_to_shapecode.get(shape_1,None)
   if s1 is None:
@@ -370,6 +413,31 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
   ipar = np.array([n1,n2,n_spots_1,n_spots_2,l1,l2,s1,s2,1,0],
       dtype=int)
 
+  if ld_1 == 'mugrid':
+      try:
+          mugrid_1 = np.array(ldc_1)
+          n_mugrid_1 = len(mugrid_1)
+      except:
+          raise Exception("failed to obtain limb darkening data from ldc_1")
+      if (n_mugrid_1 < 2):
+          raise Exception("ldc_1 must have at least 2 elements")
+  else:
+      mugrid_1 = np.array([0.])
+      n_mugrid_1 = 0
+
+  if ld_2 == 'mugrid':
+      try:
+          mugrid_2 = np.array(ldc_2)
+          n_mugrid_2 = len(mugrid_2)
+      except:
+          raise Exception("failed to obtain limb darkening data from ldc_2")
+      if (n_mugrid_2 < 2):
+          raise Exception("ldc_2 must have at least 2 elements")
+  else:
+      mugrid_2 = np.array([0.])
+      n_mugrid_2 = 0
+
+
   # Copy binary parameters into an np.array
 
   if (radius_1 < 0) or (radius_1 > 1):
@@ -382,14 +450,13 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
   if (radius_2 == 1) and (shape_2 != "roche"):
     raise ValueError("radius_2=1 only allowed for Roche potential")
 
-  par = np.zeros(37)
+  par = np.zeros(39)
   par[0] = t_zero
   par[1] = period
   par[2] = sbratio
   par[3] = radius_1
   par[4] = radius_2
   par[5] = incl
-  # par[6] = light_3
 
   if a is not None : par[7] = a
 
@@ -413,19 +480,24 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
     "claret" :  4,
     "log"    :  2,
     "sqrt"   :  2,
-    "exp"    :  2
+    "exp"    :  2,
+    "power-2":  2,
+    "mugrid" :  0 
   }
-  ld_n_1 = ldstr_to_ldcode.get(ldstr_1,None)
-  try:
-    par[11:11+ld_n_1] = ldc_1
+  ld_n_1 = ld_to_n.get(ldstr_1,None)
+  try: 
+      if (ld_n_1 > 0): 
+          par[11:11+ld_n_1] = ldc_1
   except:
     raise Exception("ldc_1 and ld_1 are inconsistent")
-  ld_n_2 = ldstr_to_ldcode.get(ldstr_2,None)
-  try:
-    par[15:15+ld_n_2] = ldc_2
+  ld_n_2 = ld_to_n.get(ldstr_2,None)
+  try: 
+      if (ld_n_2 > 0): 
+          par[15:15+ld_n_2] = ldc_2
   except:
     raise Exception("ldc_2 and ld_2 are inconsistent")
 
+
   if gdc_1 is not None : par[19] = gdc_1
 
   if gdc_2 is not None : par[20] = gdc_2
@@ -460,6 +532,30 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
     else:
       raise Exception('Invalid size for array heat_2')
 
+  if lambda_1 is not None : par[33] = lambda_1
+
+  if lambda_2 is not None : par[34] = lambda_2
+
+  if vsini_1 is not None : par[35] = vsini_1
+
+  if vsini_2 is not None : par[36] = vsini_2
+
+  if shape_1 == "love":
+    if rotfac_1 != 1:
+      raise Exception(
+                'Non-synchronous rotation not implemented for shape_1="love"')
+    if (hf_1 <= 2/3.) or (hf_1 > 5):
+        raise Exception('Invalid value for hf_1')
+    par[37] = hf_1
+
+  if shape_2 == "love":
+    if rotfac_2 != 1:
+      raise Exception(
+                'Non-synchronous rotation not implemented for shape_2="love"')
+    if (hf_2 <= 2/3.) or (hf_2 > 5):
+        raise Exception('Invalid value for hf_2')
+    par[38] = hf_2
+
   t_obs_array = np.array(t_obs)
   n_obs = len(t_obs_array)
   if t_exp is None:
@@ -493,30 +589,27 @@ def fluxes(t_obs, radius_1, radius_2, sbratio, incl,
       else:
         w_calc = np.append(w_calc, np.ones_like(t_obs_i)/(i_int-1.))
 
-  verbose = 1
+  lc_rv_flags = ellc_f.ellc.lc(t_calc,par,ipar,spar_1,spar_2,
+          n_mugrid_1, mugrid_1,n_mugrid_2,mugrid_2, verbose)
 
-  lc_rv_flags = ellc_f.ellc.lc(t_calc,par,ipar,spar_1,spar_2,verbose)
+  if (np.sum(np.isnan(lc_rv_flags)) > 0 ) & (verbose > 0):
+    lc_dummy = ellc_f.ellc.lc(t_calc[j],par,ipar,spar_1,spar_2,
+               n_mugrid_1, mugrid_1,n_mugrid_2, mugrid_2,9)
 
   flux1 = np.zeros(n_obs)
-  for j in range(0,len(t_calc)):
-    flux1[i_calc[j]] += lc_rv_flags[j,1]*w_calc[j]
-  t_obs_0 = t_obs_array[n_int_array == 0 ] # Points to be interpolated
-  n_obs_0 = len(t_obs_0)
-  if n_obs_0 > 0 :
-    i_sort = np.argsort(t_calc)
-    t_int = t_calc[i_sort]
-    f_int = lc_rv_flags[i_sort,0]
-    flux1[n_int_array == 0 ] = np.interp(t_obs_0,t_int,f_int)
-
   flux2 = np.zeros(n_obs)
   for j in range(0,len(t_calc)):
+    flux1[i_calc[j]] += lc_rv_flags[j,1]*w_calc[j]
     flux2[i_calc[j]] += lc_rv_flags[j,2]*w_calc[j]
+
   t_obs_0 = t_obs_array[n_int_array == 0 ] # Points to be interpolated
   n_obs_0 = len(t_obs_0)
   if n_obs_0 > 0 :
     i_sort = np.argsort(t_calc)
     t_int = t_calc[i_sort]
-    f_int = lc_rv_flags[i_sort,0]
+    f_int = lc_rv_flags[i_sort,1]
+    flux1[n_int_array == 0 ] = np.interp(t_obs_0,t_int,f_int)
+    f_int = lc_rv_flags[i_sort,2]
     flux2[n_int_array == 0 ] = np.interp(t_obs_0,t_int,f_int)
 
   return flux1,flux2
